package com.qmrz.generator;

import com.qmrz.exception.ABException;
import com.qmrz.service.TableService;
import com.qmrz.utils.CGHelper;
import com.qmrz.utils.Fn;
import com.qmrz.utils.MapUtil;
import com.qmrz.utils.SpringContextUtil;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Mapper生成
 */
public class MapperGenerator {
    private TableService tableService;
    private String dbName;

    public MapperGenerator(String dbName) {
        tableService = SpringContextUtil.getBean(TableService.class);
        this.dbName = dbName;
    }

    public void generator() {

        List<Map> tableList = tableService.getTableList(dbName);
        List<Map> columnList = tableService.getColumnList(dbName);

        Map<String, List<Map>> result = columnList.stream().collect(Collectors.groupingBy(map -> Fn.toString(map.get("table_name")), Collectors.toList()));

        CGHelper cgHelper = new CGHelper("mapper2.ftl", dbName);

        tableList.forEach(item -> {
            String table_name = item.get("table_name").toString();
            String table_name2 = Fn.firstUpperCase(table_name);

            List<Map> tmpColumnList = result.get(table_name);

            List<String> orderByList = new ArrayList<>();
            tmpColumnList.stream().filter(i -> {
                String tmpColumnName = Fn.toString(i.get("column_name"));
                if ("createdatetime".equals(tmpColumnName)) {
                    orderByList.add("t."+tmpColumnName + " desc");
                    return true;
                }
                return false;
            }).findFirst();

            StringBuilder primaryKeyName = new StringBuilder();

            tmpColumnList.stream().filter(i -> {
                if ("PRI".equals(Fn.toString(i.get("column_key")))) {
                    orderByList.add("t."+Fn.toString(i.get("column_name")) + " desc");
                    primaryKeyName.append(Fn.toString(i.get("column_name")));
                    return true;
                }
                return false;
            }).findFirst();

            if (primaryKeyName.length() == 0) {
                throw new ABException("表 " + table_name + " 没有主键");
            }

            String orderBy = "";

            if (orderByList.size() > 0) {
                orderBy = "order by " + String.join(",", orderByList);
            }

            item.put("columnList", tmpColumnList);
            item.put("dbname", dbName);
            item.put("table_name2", table_name2);//首字母大写
            item.put("orderBy", orderBy);
            item.put("primaryKeyName", primaryKeyName.toString());
            cgHelper.generator("mapper2", table_name2 + "Mapper.xml", item);
        });
    }
}
