<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN"
        "http://mybatis.org/dtd/mybatis-3-mapper.dtd">

<!--
${table_name}（${table_comment}）
注意：分隔线以下 不增加业务，有新增自定义业务时加到分隔线上面
 -->
<mapper namespace="com.qmrz.domain.mapper.${table_name2}Mapper">



    <!-- ========================= 分隔线，以下为生成的通用代码 ========================= -->
    <!-- 通用查询条件 -->
    <sql id="common_where">
        <where>
        <#list columnList as item>
            <if test="${item.column_name}!=null and ${item.column_name}!=''">and a.${item.column_name} = ${"#"}{${item.column_name}}</if>
        </#list>

        <#list columnList as item>
            <if test="like_${item.column_name}!=null and like_${item.column_name}!=''">and a.${item.column_name} like concat('%',${"#"}{like_${item.column_name}},'%')</if>
        </#list>
        </where>
    </sql>
    <!-- 通用查询记录 -->
    <select id="selectList" resultType="java.util.LinkedHashMap">
        select <#list columnList as item>a.${item.column_name}<#if item_has_next>,</#if></#list>
        from ${table_name} a
        <include refid="common_where" />
        <if test="__limit__!=null">
        <#noparse>limit #{__limit__}</#noparse>
        </if>
    </select>

    <!-- 通用查询记录总数 -->
    <select id="selectCount" resultType="Integer">
    select count(*) from ${table_name} a
    <include refid="common_where" />
    </select>

    <!-- 通用插入数据 -->
    <insert id="insert">
        insert into ${table_name}(
        <trim suffixOverrides=",">
            <#list columnList as item>
            <if test="${item.column_name}!=null">${item.column_name},</if>
            </#list>
        </trim>)
        values(
        <trim suffixOverrides=",">
            <#list columnList as item>
            <if test="${item.column_name}!=null">${"#"}{${item.column_name}},</if>
            </#list>
        </trim>
        )
    </insert>

    <!-- 通用更新数据 -->
    <update id="update">
        update ${table_name}
        <set>
            <#list columnList as item>
                <#if item.column_key?? && item.column_key!="PRI" || item.column_name?? && item.column_name != "id">
            <if test="${item.column_name}!=null">${item.column_name} = ${"#"}{${item.column_name}},</if>
                </#if>
            </#list>
        </set>
        <where>
            <#list columnList as item>
            <if test="where_${item.column_name}!=null">and ${item.column_name} = ${"#"}{where_${item.column_name}}</if>
            </#list>
        </where>
    </update>

    <!-- 通用删除记录 -->
    <delete id="delete">
        delete from ${table_name} a
        <include refid="common_where"/>
    </delete>

    <!-- ============================== 以下为api同步使用 ============================== -->
    <select id="sync_batch_select" resultType="Map" parameterType="Map">
        select <#list columnList as item>${item.column_name}<#if item_has_next>,</#if></#list>
        from ${table_name}
    </select>

    <insert id="sync_batch_insert" parameterType="Map">
        insert into ${table_name}(<#list columnList as item>${item.column_name}<#if item_has_next>,</#if></#list>)
        values
        <foreach collection="list" item="item" index="index" separator=",">
            (<#list columnList as item>${"#"}{item.${item.column_name}}<#if item_has_next>,</#if></#list>)
        </foreach>
    </insert>

    <update id="sync_batch_update" parameterType="List">
        <foreach collection="list" item="item" index="index" separator=";">
            update ${table_name}
            <set>
                <#list columnList as item>
                    <#if item.column_key?? && item.column_key!="PRI" && item.column_name?? && item.column_name != "id" && item.column_name!="createtime">
                ${item.column_name}=${"#"}{item.${item.column_name}},
                    </#if>
                </#list>
            </set>
            where id=${"#"}{item.id}
        </foreach>
    </update>

    <delete id="sync_batch_delete" parameterType="List">
        delete from ${table_name} where id in (
        <foreach collection="list" item="item" index="index" separator=",">
        ${"#"}{item}
        </foreach>
        )
    </delete>
</mapper>