<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN"
        "http://mybatis.org/dtd/mybatis-3-mapper.dtd">

<!--
${table_name}（${table_comment}）
注意：分隔线以下 不增加业务，有新增自定义业务时加到分隔线上面
 -->
<mapper namespace="com.qmrz.domain.mapper.${table_name2}Mapper">


    <!-- ========================= 分隔线以下为生成的通用代码 ========================= -->
    <!--
     注意：
        1、增加条件必须增加 _parameter.containsKey
        2、修改字段手动更改此文件，不推荐重新生成后全部覆盖
     -->

    <!-- 通用查询条件 -->
    <sql id="common_where">
        <#list columnList as item>
            <if test="_parameter.containsKey('where_${item.column_name}')">and t.${item.column_name} = ${"#"}{where_${item.column_name}}</if>
        </#list>

        <#list columnList as item>
            <if test="_parameter.containsKey('like_${item.column_name}')">and t.${item.column_name} like concat('%',${"#"}{like_${item.column_name}},'%')</if>
        </#list>

        <if test="_parameter.containsKey('list_${primaryKeyName}')">
            and t.${primaryKeyName} in
            <foreach collection="list_${primaryKeyName}" item="item" open="(" separator="," close=")">
            ${"#"}{item}
            </foreach>
        </if>
    </sql>
    <!-- 通用查询记录 -->
    <select id="selectList" resultType="java.util.LinkedHashMap">
        select <#list columnList as item>t.${item.column_name}<#if item_has_next>,</#if></#list>
        from ${table_name} t
        <where>
            <include refid="common_where" />
        </where>
    ${orderBy}
        <if test="_parameter.containsKey('__limit__')">
        <#noparse>limit #{__limit__}</#noparse>
        </if>
    </select>

    <!-- 通用查询记录总数 -->
    <select id="selectCount" resultType="Integer">
        select count(0) as cou from ${table_name} t
        <where>
            <include refid="common_where" />
        </where>
    </select>

    <!-- 通用插入数据 -->
    <insert id="insert">
        insert into ${table_name}(
        <trim suffixOverrides=",">
            <#list columnList as item>
                <if test="_parameter.containsKey('${item.column_name}')">${item.column_name},</if>
            </#list>
        </trim>)
        values(
        <trim suffixOverrides=",">
            <#list columnList as item>
                <if test="_parameter.containsKey('${item.column_name}')">${"#"}{${item.column_name}},</if>
            </#list>
        </trim>
        )
    </insert>

    <!-- 通用更新数据 -->
    <update id="update">
        update ${table_name} t
        <set>
            <#list columnList as item>
                <#if item.column_key?? && item.column_key!="PRI" || item.column_name?? && item.column_name != "id">
            <if test="_parameter.containsKey('${item.column_name}')">t.${item.column_name} = ${"#"}{${item.column_name}},</if>
                </#if>
            </#list>
        </set>
        <where>
            <include refid="common_where" />
        </where>
    </update>

    <!-- 通用删除记录 -->
    <delete id="delete">
        delete t from ${table_name} t
        <where>
            <include refid="common_where" />
        </where>
    </delete>
</mapper>