package com.qmrz.admin;

import com.qmrz.constants.ConstantValue;
import com.qmrz.dto.SecurityAdminDTO;
import com.qmrz.exception.ABException;
import com.qmrz.utils.Session;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;

/**
 * 管理员登录信息获取
 */
public class AdminLogin {
    public static final AdminLogin instance = new AdminLogin();

    /**
     * 获取管理员登录信息
     *
     * @return
     */
//    public AdminLoginInfo getAdminInfo() {
//        AdminLoginInfo info = Session.getSession("admininfo");
//        return info;
//    }
    public SecurityAdminDTO getAdminInfo() {
        Subject currentUser = SecurityUtils.getSubject();
        SecurityAdminDTO info = (SecurityAdminDTO) currentUser.getPrincipal();

        return info;
    }

    public Boolean isSuper() {
        return ConstantValue.ONE.equals(getAdminInfo().getIssuper());
    }

    /**
     * 获取管理员名称
     *
     * @return
     */
//    public String getAdminName() {
//        AdminLoginInfo info = getAdminInfo();
//        if (info == null) {
//            return "未登录";
//        }
//        return info.getAdminname();
//    }
    public String getAdminName() {
        Subject currentUser = SecurityUtils.getSubject();
        if (!currentUser.isAuthenticated()) {
            return "未登录";
        }
        return ((SecurityAdminDTO) currentUser.getPrincipal()).getUsername();
    }

    /**
     * 验证用户是否登录
     *
     * @return
     */
//    public boolean isLogin() {
//        AdminLoginInfo info = getAdminInfo();
//        if (info == null) {
//            return false;
//        }
//        return info.getAdminid() > 0;
//    }
    public boolean isLogin() {
        Subject currentUser = SecurityUtils.getSubject();
        return currentUser.isAuthenticated();
    }

    public static final String CustomerInfo_SessionName = "admin_customer_info";

    public static CustomerInfo getCustomerInfo() {
        CustomerInfo info = Session.getSession(CustomerInfo_SessionName);
        if (info == null) {
            throw new ABException("客户标识不存在");
        }

        if (info.getCustomerID() == null || info.getCustomerDir() == null) {
            throw new ABException("客户id 或 dir为空");
        }

        return info;
    }

    public static Integer getCustomerID() {
        return getCustomerInfo().getCustomerID();
    }

    public static void setCustomInfo(CustomerInfo customInfo){
        if (customInfo.getCustomerID() == null || customInfo.getCustomerDir() == null) {
            throw new ABException("客户id 或 dir为空");
        }

        Session.setSession(CustomerInfo_SessionName,customInfo);
    }
}
