package com.qmrz.config;

import com.github.pagehelper.PageInterceptor;
import com.qmrz.mybatis.MapperScannerConfigurer;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.sql.DataSource;

/**
 * 初始配置
 */
@Slf4j
@Configuration
public class Config {
    public Config() {
        log.info("------------ Config");
    }

    /**
     * 配置数据源
     *
     * @param dbInfo 数据库配置信息
     * @return
     */
    @Bean
    public DataSource dataSource(ConfigDBInfo dbInfo) {
        log.info("------------ dataSource");
        log.info(dbInfo.toString());
        return ConfigDBUtils.getDruid(dbInfo);
    }

    /**
     * 利用SqlSessionFactoryBean，绑定mapper(xml)位置
     *
     * @param dataSource 数据源
     * @return
     */
    @Bean("SqlSessionFactory")
    public SqlSessionFactoryBean sqlSessionFactoryBean(DataSource dataSource,PageInterceptor pageHelper) throws Exception {
        log.info("------------ SqlSessionFactoryBean");
        SqlSessionFactoryBean sqlSessionFactoryBean =
                ConfigDBUtils.getSqlSessionFactoryBean(dataSource, "classpath*:mapper/*.xml",pageHelper);

        return sqlSessionFactoryBean;
    }

    @Bean("sqlSessionTemplate")
    public SqlSessionTemplate sqlSessionTemplate(SqlSessionFactory sqlSessionFactory) {
        log.info("------------ sqlSessionTemplate");
        SqlSessionTemplate s = new SqlSessionTemplate(sqlSessionFactory);
        return s;
    }

    /**
     * mapper配置扫描并绑定
     *
     * @return
     */
    @Bean
    public MapperScannerConfigurer mapperScannerConfigurer() {
        log.info("------------ MapperScannerConfigurer");
        //SqlSessionFactory名称即是SqlSessionFactoryBean的指定bean名
        return ConfigDBUtils.getMapperScannerConfigurer("SqlSessionFactory", "com.qmrz.domain.mapper");
    }

    @Bean
    public PageInterceptor pageHelper() {
        return ConfigDBUtils.pageHelper();
    }
}
