package com.qmrz.exception;

import com.qmrz.utils.RD;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@ControllerAdvice
public class GlobalControllerException extends ResponseEntityExceptionHandler {

    private ObjectError error;

    @Override
    protected ResponseEntity<Object> handleBindException(BindException ex, HttpHeaders headers, HttpStatus status, WebRequest request) {
        log.info("------ GlobalControllerException -----");
        //ex.printStackTrace();
        //System.out.println("====== GlobalControllerException =====");

        List<ObjectError> list = ex.getAllErrors();

        if (list.size() > 0) {
            return handleExceptionInternal(ex, RD.failure(list.get(0).getDefaultMessage()), headers, status, request);
        }

        return handleExceptionInternal(ex, RD.failure(ex.getMessage()), headers, status, request);
    }

    @Override
    protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex, HttpHeaders headers, HttpStatus status, WebRequest request) {
        System.out.println("------ MethodArgumentExceptionHandler -----");
        //ex.printStackTrace();
        //System.out.println("====== MethodArgumentExceptionHandler =====");

        List<ObjectError> list = ex.getBindingResult().getAllErrors();

        if (list.size() > 0) {
//            return handleExceptionInternal(ex, RD.failure(list.get(0).getDefaultMessage()), headers, status, request);
            Map<String, String> result = new HashMap<>();
            for (ObjectError error : list) {
                if (error instanceof FieldError) {
                    FieldError err = (FieldError) error;
                    result.put(err.getField(), err.getDefaultMessage());
                }
            }

            if (result.size() > 0) {
                return handleExceptionInternal(ex, RD.failureByForm("操作失败", result), headers, HttpStatus.OK, request);
            }
        }

        return handleExceptionInternal(ex, RD.failure(ex.getMessage()), headers, status, request);
    }
}
