package com.qmrz.mybatis;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.qmrz.dto.ParamPageDTO;
import com.qmrz.exception.ABException;
import com.qmrz.mybatis.service.BaseService;
import com.qmrz.mybatis.sql.BaseWrapper;
import com.qmrz.mybatis.sql.sqlnode.UpdateParam;
import com.qmrz.utils.PageData;
import com.qmrz.utils.SpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.SqlSessionTemplate;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

@Slf4j
public class BasePO<T> implements BaseService<T> {
    SqlSessionTemplate sqlTemp;
    POUtil poUtil;
    private POProxy poProxy;

    public void _setPoProxy(POProxy poProxy) {
        this.poProxy = poProxy;
    }

    public POProxy _getProxy() {
        return poProxy;
    }

    public void _setPoUtil(POUtil poUtil) {
        this.poUtil = poUtil;
    }

    public BasePO() {
        this.sqlTemp = SpringContextUtil.getBean(SqlSessionTemplate.class);
    }

    private String getMSID(String methodName) {
        String claName;
        if (poProxy != null) {
            claName = poProxy.getClassPO().getName();
        } else {
            claName = this.getClass().getName();
        }
        return claName + "_" + methodName + "_ActiveRecord";
    }

    private Map<String, Object> toMap(Object obj) {
        return toMap(obj, true);
    }

    private Map<String, Object> toMap(Object obj, boolean isContainKey) {
        if (obj instanceof Map) {
            return (Map) obj;
        }

        if (obj instanceof BasePO) {
            BasePO basePO = (BasePO) obj;
            if (basePO._getProxy() == null) {
                throw new ABException("请使用proxy()创建实体对象:" + this.getClass().getName());
//                return new POUtil(obj.getClass()).getMapNotNull(obj);
            } else {

                Map result = basePO._getProxy().getSetList();
                if (!isContainKey) {
                    result.remove(poUtil.getPrimarykey().getFieldNameByDB());
                }
                return result;
            }
        } else {
            return poUtil.getParam(obj);
        }

//        throw new ABException("非法参数");
    }

    private BasePO createProxy() {
        poProxy = new POProxy();
        poUtil = new POUtil(this.getClass());

        BasePO po = poProxy.getProxy(this.getClass());

        po._setPoProxy(poProxy);
        po._setPoUtil(poUtil);
        return po;
    }

    /**
     * 创建代理对象
     *
     * @return
     */
    public T proxy() {
        return (T) this.createProxy();
    }

    /**
     * 创建代理对象，并初始化填充代理对象
     *
     * @param object 将object属性填充到代理对象（即将非代理对象使用代理对象接管）
     * @return
     */
    public T proxy(Object object) {
        return proxy(object, false);
    }

    /**
     * @param object
     * @param isAllowNull 指定object里属性是否允许为空
     * @return
     */
    public T proxy(Object object, boolean isAllowNull) {
        BasePO po = this.createProxy();
        POUtil.fillNotNull(po, object, isAllowNull);
        return (T) po;
    }

    /**
     * 创建代理对象，并初始化填充代理对象数据
     *
     * @param object
     * @param only   初始指定字段（初始化数据时，只填充在onlyFieldNameList里包含的字段）
     * @return
     */
    public T proxy(Object object, String... only) {
        return proxy(object, false, only);
    }

    public T proxy(Object object, boolean isAllowNull, String... only) {
        BasePO po = this.createProxy();

        //将object属性填充到代理对象（即将非代理对象使用代理对象接管）
        POUtil.fillNotNull(po, object, only, isAllowNull);
        return (T) po;
    }

    @Override
    public List<T> selectWrapper(Consumer<? super BaseWrapper> wrapper) {
        return this.sqlTemp.selectList(getMSID("selectWrapper"), wrapper);
    }

    @Override
    public T selectOneWrapper(Consumer<? super BaseWrapper> wrapper) {
        List<T> list = this.sqlTemp.selectList(getMSID("selectOneWrapper"), wrapper);
        if (list.size() == 0) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public Integer updateWrapper(Object param, Consumer<? super BaseWrapper> wrapper) {
        return this.sqlTemp.update(getMSID("updateWrapper")
                , new UpdateParam(toMap(param), wrapper));
    }

    public Integer update(Consumer<? super BaseWrapper> wrapper) {
        return this.sqlTemp.update(getMSID("updateWrapper")
                , new UpdateParam(toMap(this), wrapper));
    }

    @Override
    public Integer deleteWrapper(Consumer<? super BaseWrapper> wrapper) {
        return this.sqlTemp.delete(getMSID("deleteWrapper"), wrapper);
    }

    public PageData<T> selectPage(ParamPageDTO paramPageDTO, Consumer<? super BaseWrapper> wrapper) {
        PageHelper.startPage(paramPageDTO.getPageIndex(), paramPageDTO.getPageSize());
        Page<T> result = (Page<T>) this.sqlTemp.selectList(getMSID("selectWrapper"), wrapper);
        return PageData.newobj(result);
    }

    public List<T> select() {
        return this.select(this);
    }

    @Override
    public List<T> select(Object whereObject) {
        return this.sqlTemp.selectList(getMSID("select"), toMap(whereObject));
    }

    public PageData<T> selectPage(ParamPageDTO paramPageDTO) {
        PageHelper.startPage(paramPageDTO.getPageIndex(), paramPageDTO.getPageSize());
        return PageData.newobj((Page<T>) (this.select(this)));
    }

    @Override
    public T selectOne(Object whereObject) {
        return this.sqlTemp.selectOne(getMSID("selectOne"), toMap(whereObject));
    }

    public T selectOne() {
        return this.selectOne(this);
    }

    @Override
    public Integer selectCount(Object whereObject) {
        List<Integer> result = this.sqlTemp.selectList(getMSID("selectCount"), toMap(whereObject));
        return result.get(0);
    }

    public Integer selectCount() {
        return this.selectCount(this);
    }

    @Override
    public List<T> selectAll() {
        return this.sqlTemp.selectList(getMSID("selectAll"), new LinkedHashMap<>());
    }

    @Override
    public T selectByPrimaryKey(Object paramPrimaryKey) {
        return this.selectOne(poUtil.getParamPrimaryKey(paramPrimaryKey));
    }

    public T selectByPrimaryKey() {
        return this.selectByPrimaryKey(this);
    }

    @Override
    public int delete(Object whereObject) {
        return this.sqlTemp.delete(getMSID("delete"), toMap(whereObject));
    }

    @Override
    public int deleteByPrimaryKey(Object paramPrimaryKey) {
        return this.delete(poUtil.getParamPrimaryKey(paramPrimaryKey));
    }

    public int deleteByPrimaryKey() {
        return this.deleteByPrimaryKey(this);
    }

    public int delete() {
        return this.delete(this);
    }

    @Override
    public int insert(T entity) {
        return this.sqlTemp.insert(getMSID("insert"), entity);
    }

    public int insert() {
        return this.insert((T) this);
    }

    public int update(Object paramSet, Object paramWhere) {
        UpdateParam param = new UpdateParam(toMap(paramSet), toMap(paramWhere));
        return this.update(param);
    }

    @Override
    public int updateByPrimaryKey(Object whereValue) {
        return this.update(toMap(this, false), poUtil.getParamPrimaryKeyByValue(whereValue));
        //poUtil.getParamWriteNotNull(this)
    }

    @Override
    public int update(UpdateParam updateParam) {
        return this.sqlTemp.update(getMSID("update"), updateParam);
    }

    public int updateByPrimaryKey() {
        return this.updateByPrimaryKey(poUtil.getValueByPrimaryKey(this));
    }

    public int updateByPrimaryKey(Object paramSet, Object whereValue) {
        return this.update(paramSet, poUtil.getParamPrimaryKeyByValue(whereValue));
    }
}
