package com.qmrz.mybatis;

import com.qmrz.mybatis.sql.sqlnode.SqlNodeWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.scripting.xmltags.SqlNode;

@Slf4j
public class BaseProviderSqlSource {
    public SqlNode selectWrapper(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "selectWrapper");
    }

    public SqlNode selectOneWrapper(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "selectOneWrapper");
    }

    public SqlNode updateWrapper(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "updateWrapper");
    }

    public SqlNode deleteWrapper(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "deleteWrapper");
    }

    /**
     * 该方法仅仅用来初始化ProviderSqlSource
     *
     * @param record
     * @return
     */
    public String dynamicSQL(Object record) {
        log.info("BaseProviderSqlSource dynamicSQL");
        return "dynamicSQL";
    }

    public SqlNode select(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "select");

        //修改返回值类型为实体类型
//        MapperUtil.setResultType(ms, classPO);
//
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.selectAllColumns());
//        sql.append(sqlUtil.fromTable());
//        sql.append(sqlUtil.whereAllIfFieldXML());
//        sql.append(sqlUtil.orderByDefault());
//
//        return sql.toString();
    }

    /**
     * 查询全部结果
     */
    public SqlNode selectAll(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "select");
        //修改返回值类型为实体类型
//        MapperUtil.setResultType(ms, classPO);
//
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.selectAllColumns());
//        sql.append(sqlUtil.fromTable());
//        sql.append(sqlUtil.orderByDefault());
//
//        return sql.toString();
    }

    public SqlNode selectOne(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "selectOne");

        //修改返回值类型为实体类型
//        MapperUtil.setResultType(ms, classPO);
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.selectAllColumns());
//        sql.append(sqlUtil.fromTable());
//        sql.append(sqlUtil.whereXML());
//        sql.append(sqlUtil.limit(1));
//        return sql.toString();
    }

    /**
     * 根据主键字段进行查询，查询条件使用等号
     */
    public SqlNode selectByPrimaryKey(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType(ms, classPO);
        return new SqlNodeWrapper(classPO, "select");

        //修改返回值类型为实体类型
//        MapperUtil.setResultType(ms, classPO);
//
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.selectAllColumns());
//        sql.append(sqlUtil.fromTable());
//        sql.append(sqlUtil.whereByPrimaryKeyXML());
//
//        return sql.toString();
    }

    /**
     * 根据实体中的属性查询总数，查询条件使用等号
     */
    public SqlNode selectCount(MappedStatement ms, Class<?> classPO) {
        MapperUtil.setResultType2(ms, Integer.class,"_count");
        return new SqlNodeWrapper(classPO, "selectCount");


        //修改返回值类型为实体类型
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.selectCount());
//        sql.append(sqlUtil.fromTable());
//        sql.append(sqlUtil.whereAllIfFieldXML());
//
//        return sql.toString();
    }

    /**
     * 根据主键字段进行删除，方法参数必须包含完整的主键属性
     */
    public SqlNode deleteByPrimaryKey(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "delete");

//        SqlUtil sqlUtil = new SqlUtil(classPO);
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.deleteFrom());
//        sql.append(sqlUtil.whereByPrimaryKeyXML());
//        return sql.toString();
    }

    /**
     * 根据实体属性作为条件进行删除，查询条件使用等号
     */
    public SqlNode delete(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "delete");

//        SqlUtil sqlUtil = new SqlUtil(classPO);
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.deleteFrom());
//        sql.append(sqlUtil.whereAllIfFieldXML());
//        return sql.toString();
    }

    /**
     * 保存一个实体，null的属性不会保存，会使用数据库默认值
     */
    public SqlNode insert(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "insert");

//        SqlUtil sqlUtil = new SqlUtil(classPO);
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.insertXML());
//        return sql.toString();
    }

//    /**
//     * 保存一个实体，所有字段都会插入，null的属性也会保存，不会使用数据库默认值
//     */
//    public String insertAllProperty(MappedStatement ms){
//        System.out.println("BaseProviderSqlSource select");
//        return "select id,name from test where 1=#{id}";
//    }

    /**
     * 根据主键更新实体全部字段，null值不会被更新
     */
    public SqlNode updateByPrimaryKey(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "update");
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.updateSetXML());
//        sql.append(sqlUtil.whereByPrimaryKeyXML());
//        return sql.toString();
    }

    public SqlNode update(MappedStatement ms, Class<?> classPO) {
        return new SqlNodeWrapper(classPO, "update");
//        SqlUtil sqlUtil = new SqlUtil(classPO);
//        StringBuilder sql = new StringBuilder();
//        sql.append(sqlUtil.updateSetXML());
//        sql.append(sqlUtil.whereByPrimaryKeyXML());
//        return sql.toString();
    }

    /**
     * 根据主键更新实体全部字段，null值也会被更新
     */
//    public String updateByPrimaryKeyAllProperty(MappedStatement ms){
//        System.out.println("BaseProviderSqlSource select");
//        return "select id,name from test where 1=#{id}";
//    }
}
