package com.qmrz.mybatis;

import com.qmrz.mybatis.sql.BaseWrapper;
import org.apache.ibatis.annotations.*;

import java.util.List;
import java.util.function.Consumer;

public interface Mapper<T> {
//    @Options(useGeneratedKeys = true, keyProperty = "id")
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    List<T> selectWrapper(Consumer<? super BaseWrapper> wrapper);

    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    T selectOneWrapper(Consumer<? super BaseWrapper> wrapper);

    @UpdateProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    Integer updateWrapper(Object param, Consumer<? super BaseWrapper> wrapper);

    @DeleteProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    Integer deleteWrapper(Consumer<? super BaseWrapper> wrapper);

    /**
     * 根据实体中的属性值进行查询，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    List<T> select(Object whereObject);

    //@SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    //List<T> selectList(T entity, IWrapper wrapper);

    /**
     * 根据实体中的属性进行查询，只能有一个返回值，有多个结果取第1个
     *
     * @param whereObject map或实体
     * @return
     */
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    T selectOne(Object whereObject);

    /**
     * 根据实体中的属性查询总数，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    Integer selectCount(Object whereObject);

    /**
     * 查询全部结果
     *
     * @return
     */
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    List<T> selectAll();

    /**
     * 根据主键字段进行查询，查询条件使用等号
     *
     * @param paramPrimaryKey map或实体
     * @return
     */
    @SelectProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    T selectByPrimaryKey(Object paramPrimaryKey);

    /**
     * 根据主键字段进行删除，方法参数必须包含完整的主键属性
     *
     * @param paramPrimaryKey map或实体
     * @return
     */
    @DeleteProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    int deleteByPrimaryKey(Object paramPrimaryKey);

    /**
     * 根据实体属性作为条件进行删除，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    @DeleteProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    int delete(Object whereObject);


    /**
     * 保存一个实体，null的属性不会保存，会使用数据库默认值
     *
     * @param entity map或实体
     * @return
     */
    @Options(useGeneratedKeys=true,keyProperty = "id")
    @InsertProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    int insert(T entity);

    /**
     * 保存一个实体，所有字段都会插入，null的属性也会保存，不会使用数据库默认值
     *
     * @param entity
     * @return
     */
//    @InsertProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
//    int insertAllProperty(T entity);

    /**
     * 根据主键更新实体全部字段，null值不会被更新
     *
     * @param paramMap map或实体
     * @return
     */
    @UpdateProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    int updateByPrimaryKey(Object paramMap);


    @UpdateProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
    int update(Object paramMap);

    /**
     * 根据主键更新实体全部字段，null值也会被更新
     *
     * @param entity
     * @return
     */
//    @UpdateProvider(type = BaseProviderSqlSource.class, method = "dynamicSQL")
//    int updateByPrimaryKeyAllProperty(T entity);


}
