package com.qmrz.mybatis;

import com.qmrz.exception.ABException;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ResultMap;
import org.apache.ibatis.mapping.ResultMapping;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.apache.ibatis.session.Configuration;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Slf4j
public class MapperUtil {
    /**
     * 获取Mapper的泛型实体
     *
     * @param ms
     * @return
     */
    public static Class<?> getMapperClass(MappedStatement ms) {
//        log.warn(ms.getResource());
        String poClassName = ms.getResource().replace(".java (best guess)", "")
                .replace("/", ".");

        //获取实体
        Class<?> classPO;
        try {
            Class<?> clazz = Class.forName(poClassName);
            classPO = (Class<?>) ((ParameterizedType) (clazz.getGenericInterfaces()[0])).getActualTypeArguments()[0];

        } catch (Exception e) {
            e.printStackTrace();
            throw new ABException("通用Mapper：" + e.getMessage());
        }

//        log.warn(classPO.getName());
        return classPO;
    }

    public static void setResultType(MappedStatement ms, Class<?> classPO) {
        Configuration configuration = ms.getConfiguration();
        MetaObject msObject = SystemMetaObject.forObject(ms);

//        List<ResultMapping> resultMappings = new ArrayList<>();
//        MetaObject msObject = SystemMetaObject.forObject(ms);
//        ResultMap.Builder builder = new ResultMap.Builder(ms.getConfiguration()
//                , classPO.getName() + "_ResultType",
//                classPO, resultMappings, true);
//
//
//
//        List<ResultMap> resultMaps = new ArrayList<>();
//        resultMaps.add(builder.build());
//        msObject.setValue("resultMaps", Collections.unmodifiableList(resultMaps));

        List<ResultMapping> resultMappings = new ArrayList<>();

        for (Field field : classPO.getDeclaredFields()) {
            ResultMapping.Builder builder = new ResultMapping.Builder(configuration
                    , field.getName(), field.getName()//, POUtil.getFieldNameByDB(field)
                    , field.getType());
            resultMappings.add(builder.build());
        }

        ResultMap.Builder builder = new ResultMap.Builder(configuration, classPO.getName() + "_CommonMapperResultMap",
                classPO, resultMappings, true);

        List<ResultMap> resultMaps = new ArrayList<>();
        resultMaps.add(builder.build());
        msObject.setValue("resultMaps", Collections.unmodifiableList(resultMaps));
    }

    public static void setResultType2(MappedStatement ms, Class<?> cla, String fieldName) {
        Configuration configuration = ms.getConfiguration();
        MetaObject msObject = SystemMetaObject.forObject(ms);
        List<ResultMapping> resultMappings = new ArrayList<>();

        ResultMapping.Builder builder2 = new ResultMapping.Builder(configuration
                , fieldName, fieldName//, POUtil.getFieldNameByDB(field)
                , cla);
        resultMappings.add(builder2.build());

        ResultMap.Builder builder = new ResultMap.Builder(configuration, cla.getName() + "_CommonMapperResultMap",
                cla, resultMappings, true);

        List<ResultMap> resultMaps = new ArrayList<>();
        resultMaps.add(builder.build());
        msObject.setValue("resultMaps", Collections.unmodifiableList(resultMaps));
    }
}
