package com.qmrz.mybatis;

import org.springframework.cglib.proxy.Enhancer;
import org.springframework.cglib.proxy.MethodInterceptor;
import org.springframework.cglib.proxy.MethodProxy;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.util.*;

/**
 * 实体代理类，用于监控实体属性的变化
 */
public class POProxy implements MethodInterceptor {
    private Enhancer enhancer = new Enhancer();

    private Class<?> classPO;
    private Map<String, Object> setList = new LinkedHashMap<>();

    public Class<?> getClassPO() {
        return classPO;
    }

    public Map<String, Object> getSetList() {
        return setList;
    }

    /**
     * 创建代理
     * @param classPO
     * @param <T>
     * @return
     */
    public <T> T getProxy(Class classPO) {
        this.classPO = classPO;
        //设置创建子类的类
        enhancer.setSuperclass(classPO);
        enhancer.setCallback(this);
        return (T) enhancer.create();
    }

    /**
     * 监控set属性
     * @param o
     * @param method
     * @param objects
     * @param methodProxy
     * @return
     * @throws Throwable
     */
    @Override
    public Object intercept(Object o, Method method, Object[] objects, MethodProxy methodProxy) throws Throwable {
        String methodName = method.getName().toLowerCase();
        if (methodName.startsWith("set")) {
            setList.put(methodName.substring(3, methodName.length()), objects[0]);
        }

        //代理类调用父类的方法
        return methodProxy.invokeSuper(o, objects);
    }
}
