package com.qmrz.mybatis;

import com.qmrz.exception.ABException;
import com.qmrz.mybatis.annotation.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.*;

public class POUtil {
    private static final Class<? extends Annotation> AnnotationID = ID.class;
    private static final Class<? extends Annotation> AnnotationDBType = DBType.class;
    private static final Class<? extends Annotation> AnnotationFieldName = FieldName.class;
    private static final Class<? extends Annotation> AnnotationTableName = TableName.class;
    private static final Class<? extends Annotation> AnnotationOrderByDefault = OrderByDefault.class;

    private Class<?> classPO;
    private String tableNameByDB;
    private String schemasNameByDB;
    private List<POPropertie> propertieList;
    private List<POPropertie> propertieListOrderBy;
    private POPropertie primarykey;

    public POUtil(Class<?> classPO) {
        this.classPO = classPO;
        this.loadTableNameByDB();
        this.loadSchemasNameByDB();
        this.loadPropertieList();
        this.loadOrderBy();
    }

    public String getTableNameByDB() {
        return tableNameByDB;
    }

    public String getSchemasNameByDB() {
        return schemasNameByDB;
    }

    public List<POPropertie> getPropertieList() {
        return propertieList;
    }

    public List<POPropertie> getPropertieListOrderBy() {
        return propertieListOrderBy;
    }

    public POPropertie getPrimarykey() {
        return primarykey;
    }

    private void loadOrderBy() {
        this.propertieListOrderBy = new ArrayList<>();
        this.propertieList.forEach(item -> {
            if (item.getOrderByStatus() != 0) {
                this.propertieListOrderBy.add(item);
            }
        });

        Collections.sort(this.propertieListOrderBy, new Comparator<POPropertie>() {
            @Override
            public int compare(POPropertie o1, POPropertie o2) {
                if (o1.getOrderByIndex() < o2.getOrderByIndex()) {
                    return -1;
                } else if (o1.getOrderByIndex() == o2.getOrderByIndex()) {
                    return 0;
                } else {
                    return 1;
                }
            }
        });
    }

    /**
     * 写入参数，insert和update时的参数，不包含主键 和 为null
     *
     * @param po
     * @return
     */
//    public Map<String, Object> getParamWriteNotNull(Object po) {
//        Map<String, Object> param = new LinkedHashMap<>();
//        getPropertieNotNull(po).forEach(item -> {
//            param.put(item.getFieldName(), item.getValue());
//        });
//        return param;
//    }

    /**
     * 实体转map，若实体对象在当前实体结构中不存在，则不加入map
     *
     * @param po
     * @return
     */
    public Map<String, Object> getParam(Object po) {
        Map<String, Object> param = new LinkedHashMap<>();
        MetaObject mo = SystemMetaObject.forObject(po);
        List<String> names = Arrays.asList(mo.getGetterNames());

        getPropertieList().forEach(item -> {
            String dbname = item.getFieldNameByDB();
            if (names.contains(dbname)) {
                param.put(dbname, mo.getValue(dbname));
            }
        });
        return param;
    }

    /**
     * @param paramPrimaryKey 实体（里面存储key字段）
     * @return
     */
    public Map<String, Object> getParamPrimaryKey(Object paramPrimaryKey) {
        MetaObject msObject = SystemMetaObject.forObject(paramPrimaryKey);
        Map<String, Object> param = new LinkedHashMap<>();
        param.put(this.primarykey.getFieldNameByDB()
                , msObject.getValue(this.primarykey.getFieldName()));
        return param;
    }

    public Map<String, Object> getParamPrimaryKeyByValue(Object value) {
        Map<String, Object> param = new LinkedHashMap<>();
        param.put(this.primarykey.getFieldNameByDB()
                , value);
        return param;
    }

    public Object getValueByPrimaryKey(Object objectPO) {
        MetaObject msObject = SystemMetaObject.forObject(objectPO);
        return msObject.getValue(this.primarykey.getFieldName());
    }

//    /**
//     * 写入参数，insert和update时的参数，不包含主键 和 为null
//     *
//     * @return
//     */
//    public Map<String, Object> getParamWriteNotNull(Object po) {
//        MetaObject msObject = SystemMetaObject.forObject(po);
//        Map<String, Object> param = new LinkedHashMap<>();
//
//        for (POPropertie item : this.getPropertieList()) {
//            if (item.isPrimarykey()) {
//                continue;
//            }
//
//            Object val = msObject.getValue(item.getFieldName());
//            if (val == null) {
//                continue;
//            }
//
//            param.put(item.getFieldName(), val);
//        }
//        return param;
//    }

    /**
     * 获取所有非null属性（不含主键、含属性值）
     *
     * @param po
     * @return
     */
//    public List<POPropertieValue> getPropertieNotNull(Object po) {
//        List<POPropertieValue> fieldList = new ArrayList<>();
//        MetaObject msObject = SystemMetaObject.forObject(po);
//
//        for (POPropertie propertie : this.propertieList) {
//            if (propertie.isPrimarykey()) {
//                continue;
//            }
//            Object fieldValue = msObject.getValue(propertie.getFieldName());
//            if (fieldValue == null) {
//                continue;
//            }
//            fieldList.add(new POPropertieValue(propertie, fieldValue));
//        }
//        return fieldList;
//    }

    /**
     * Object转map
     *
     * @param po
     * @return
     */
    public static Map<String, Object> getMap(Object po) {
        List<POPropertieValue> fieldList = new ArrayList<>();
        MetaObject msObject = SystemMetaObject.forObject(po);
        Field[] fields = po.getClass().getDeclaredFields();
        Map<String, Object> result = new LinkedHashMap<>();
        for (Field field : fields) {
            result.put(field.getName(), msObject.getValue(field.getName()));
        }

        return result;
    }

//    public Map<String, Object> getMapNotNull(Object po) {
//        List<POPropertieValue> fieldList = new ArrayList<>();
//        MetaObject msObject = SystemMetaObject.forObject(po);
//
//        Map<String, Object> result = new LinkedHashMap<>();
//        for (POPropertie item : this.propertieList) {
//            Object val = msObject.getValue(item.getFieldName());
//            if (val == null) {
//                continue;
//            }
//            if (item.isPrimarykey()) {
//                continue;
//            }
//            result.put(item.getFieldName(), val);
//        }
//
//        return result;
//    }

    /**
     * 填充实体数据，将object 属性填充到 objectProxy
     *
     * @param objectProxy
     * @param object
     */
    public static void fillNotNull(BasePO objectProxy, Object object, boolean isAllowNull) {
        List<POPropertieValue> fieldList = new ArrayList<>();
        MetaObject msObjectProxy = SystemMetaObject.forObject(objectProxy);
        MetaObject msObject = SystemMetaObject.forObject(object);

        for (Field field : objectProxy._getProxy().getClassPO().getDeclaredFields()) {
            if (!msObject.hasGetter(field.getName())) {
                continue;
            }

            Object obj = msObject.getValue(field.getName());
            if (obj == null) {
                if (isAllowNull) {
                    continue;
                } else {
                    //若object有属性，则必须不能为null
                    throw new ABException(field.getName() + "不允许为空");
                }
            }
            msObjectProxy.setValue(field.getName(), obj);
        }
    }

    public static void fillNotNull(BasePO objectProxy, Object object, String[] fieldNameList, boolean isAllowNull) {
        List<POPropertieValue> fieldList = new ArrayList<>();
        MetaObject msObjectProxy = SystemMetaObject.forObject(objectProxy);
        MetaObject msObject = SystemMetaObject.forObject(object);

        for (String fieldName : fieldNameList) {
            if (!msObject.hasGetter(fieldName)) {
                continue;
            }

            Object obj = msObject.getValue(fieldName);
            if (obj == null) {
                if (isAllowNull) {
                    continue;
                } else {
                    //若object有属性，则必须不能为null
                    throw new ABException(fieldName + "不允许为空");
                }
            }
            msObjectProxy.setValue(fieldName, obj);
        }
    }

    /**
     * 只返回指定参数
     *
     * @param po
     * @param names
     * @return
     */
    public static Map<String, Object> getMapByNames(Object po, String... names) {
        Map<String, Object> result = new LinkedHashMap<>();
        if (po == null) {
            return result;
        }
        MetaObject msObject = SystemMetaObject.forObject(po);
        for (String name : names) {
            result.put(name, msObject.getValue(name));
        }
        return result;
    }

    /**
     * 只返回指定参数
     *
     * @param list
     * @param names
     * @return
     */
    public static List<Map<String, Object>> getListMapByNames(List list, String... names) {
        List<Map<String, Object>> result = new ArrayList<>();
        for (Object o : list) {
            result.add(getMapByNames(o, names));
        }
        return result;
    }

    /**
     * 获取所有属性（不含值）
     *
     * @return
     */
    public void loadPropertieList() {
        this.propertieList = new ArrayList<>();
        Field[] fields = classPO.getDeclaredFields();
//        POPropertie propID = null;
        POPropertie propIDTmp = null;
        boolean hasOrderBy = false;//是否有指定排序注解，若没有默认主键降序排序
        for (Field field : fields) {
            String fieldName = field.getName();
            String fieldNameByDB = getFieldNameByDB(field);

            //获取order by
            int orderByStatus = 0;
            int orderByIndex = 0;
            if (field.isAnnotationPresent(AnnotationOrderByDefault)) {
                OrderByDefault orderByDefault
                        = ((OrderByDefault) field.getAnnotation(AnnotationOrderByDefault));
                boolean isasc = orderByDefault.isasc();
                if (isasc) {
                    orderByStatus = 1;
                } else {
                    orderByStatus = 2;
                }
                orderByIndex = orderByDefault.index();
                hasOrderBy = true;
            }

            //验证是否主键
            boolean isPrimarykey = false;
            if (field.isAnnotationPresent(AnnotationID)) {
                if (this.primarykey != null) {
                    throw new ABException("发现有多个 @ID 注释");
                }
                isPrimarykey = true;
            }

            String dbType = null;
            if (field.isAnnotationPresent(AnnotationDBType)) {
                dbType = ((DBType) field.getAnnotation(AnnotationDBType)).type();
            }

            POPropertie p = new POPropertie(fieldName, fieldNameByDB, field, field.getType(), dbType
                    , orderByStatus, orderByIndex, isPrimarykey);

            if (isPrimarykey) {
                this.primarykey = p;
            }

            if ("id".equals(fieldName)) {
                p.setPrimarykey(true);
                propIDTmp = p;
            }

            this.propertieList.add(p);
        }

        if (this.primarykey == null) {
            if (propIDTmp == null) {
                if (tableNameByDB.length() >= 4 && "view".equals(tableNameByDB.substring(tableNameByDB.length() - 4))) {
                    this.primarykey = new POPropertie(null, null, null, null, null, 1, 1, false);
                } else {
                    throw new ABException("必须指定主键:" + this.classPO.getName());
                }
            }else{
                this.primarykey = propIDTmp;
            }
        }

        //若没有指定排序，则按主键降序排序
        if (!hasOrderBy) {
            this.primarykey.setOrderByStatus(2);
            this.primarykey.setOrderByIndex(0);
        }
    }

    public POPropertie getPropertieByName(String name) {
        for (POPropertie item : this.propertieList) {
            if (name.equals(item.getFieldName())) {
                return item;
            }
        }
        throw new ABException("找不到字段：" + this.classPO.getName() + ":" + name);
    }

    /**
     * 获取数据库字段名
     *
     * @param field
     * @return
     */
    public static String getFieldNameByDB(Field field) {
        String fieldNameByDB;
        if (field.isAnnotationPresent(AnnotationFieldName)) {
            fieldNameByDB = ((FieldName) field.getAnnotation(AnnotationFieldName)).name();
        } else {
            fieldNameByDB = field.getName();
        }
        return fieldNameByDB;
    }

//    private static boolean getFieldOrderyByDefault(Field field) {
//        if (!field.isAnnotationPresent(AnnotationOrderByDefault)) {
//            return false;
//        }
//
//        boolean isasc = ((OrderByDefault) field.getAnnotation(AnnotationOrderByDefault)).isasc();
//        return getFieldNameByDB(field) + (isasc ? " asc" : " desc");
//    }

    /**
     * 获取数据库表名
     *
     * @return
     */
    private void loadTableNameByDB() {
        if (classPO.isAnnotationPresent(AnnotationTableName)) {
            this.tableNameByDB = ((TableName) classPO.getAnnotation(AnnotationTableName)).name();
            return;
        }

        String tableName = classPO.getSimpleName().toLowerCase();
        if (tableName.lastIndexOf("po") > 0) {
            tableName = tableName.substring(0, tableName.length() - 2);
        }

        this.tableNameByDB = tableName;
    }

    private void loadSchemasNameByDB() {
        if (classPO.isAnnotationPresent(AnnotationTableName)) {
            this.schemasNameByDB = ((TableName) classPO.getAnnotation(AnnotationTableName)).schemasName();
            return;
        }
        this.schemasNameByDB = "";
    }

//    /**
//     * 是否有该注解
//     *
//     * @param annotationClass
//     * @return
//     */
//    public boolean isAnnotationPresent(Class<? extends Annotation> annotationClass) {
//        boolean result = false;
//        if (field != null) {
//            result = field.isAnnotationPresent(annotationClass);
//        }
//        if (!result && setter != null) {
//            result = setter.isAnnotationPresent(annotationClass);
//        }
//        if (!result && getter != null) {
//            result = getter.isAnnotationPresent(annotationClass);
//        }
//        return result;
//    }
}
