package com.qmrz.mybatis;


import com.qmrz.exception.ABException;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class SqlUtil {

    private PO po;
    private POUtil poUtil;

    public SqlUtil(Class<?> entityClass) {

        po = new PO();
        poUtil = new POUtil(entityClass);

        po.setEntityClass(entityClass);
        po.setFieldList(poUtil.getPropertieList());
        po.setTableNameByDB(poUtil.getTableNameByDB());
        po.setSchemasName(poUtil.getSchemasNameByDB());

        if (po.getFieldList().size() == 0) {
            throw new ABException("没有字段");
        }
    }

    public StringBuilder selectAllColumns() {
        StringBuilder sql = new StringBuilder();
        List<String> list = new ArrayList<>();
        sql.append("select ");

        po.getFieldList().forEach(
                item -> list.add(fieldDBSelect(item))
        );
        sql.append(StringUtils.join(list.toArray(), ","));
        return sql;
    }

    public StringBuilder selectCount(String fieldName) {
        StringBuilder sql = new StringBuilder();
        sql.append("select count(").append(fieldName).append(") as _count");
        return sql;
    }

    public StringBuilder selectCount() {
        return selectCount(poUtil.getPrimarykey().getFieldNameByDB());
    }

//    private String getAllFieldStr() {
//        List<POPropertie> properties = po.getFieldList();
//        StringBuilder sql = new StringBuilder();
//        properties.forEach(
//                item -> sql.append(item.getFieldNameByDB()).append(",")
//        );
//
//        return sql.substring(0, sql.length() - 1);
//    }

    private String getOrderByStr() {
        StringBuilder sql = new StringBuilder();

        poUtil.getPropertieListOrderBy().forEach(item -> {
            int orderByStatus = item.getOrderByStatus();
            String str = "";
            if (orderByStatus == 1) {
                str = " asc";
            } else if (orderByStatus == 2) {
                str = " desc";
            }
            if (str.length() > 0) {
                sql.append(item.getFieldNameByDB()).append(str).append(",");
            }
        });

        if (sql.length() > 0) {
            return sql.substring(0, sql.length() - 1);
            //return " order by " + str;
        }

        return "";
    }

    public StringBuilder fromTable() {
        StringBuilder sql = new StringBuilder();
        sql.append(" from ");
        sql.append(po.getTableNameByDBAll());
        return sql;
    }

    public String whereByPrimaryKeyXML() {
        StringBuilder sql = new StringBuilder();
        sql.append(" <where>");

        List<POPropertie> properties = po.getFieldList();

        sql.append(poUtil.getPrimarykey().getFieldNameByDB()
                + "=#{" + poUtil.getPrimarykey().getFieldName() + "}");

        sql.append("</where>");
        return sql.toString();
    }

    public String whereByPrimaryKey() {
        StringBuilder sql = new StringBuilder();

        List<POPropertie> properties = po.getFieldList();
        sql.append(" where ");
        sql.append(poUtil.getPrimarykey().getFieldNameByDB()
                + "=#{" + poUtil.getPrimarykey().getFieldName() + "}");

        return sql.toString();
    }

    public String whereNotNullXML() {
        StringBuilder sql = new StringBuilder();
        sql.append(" <where>");

        List<POPropertie> properties = po.getFieldList();

        for (POPropertie column : properties) {
            sql.append(ifNotNullXML(column, column.getFieldNameByDB() + "=#{" + column.getFieldName() + "}"));
        }

        sql.append("</where>");
        return sql.toString();
    }

    public String whereXML() {
        StringBuilder sql = new StringBuilder();
        sql.append(" <where>");

        List<POPropertie> properties = po.getFieldList();

        for (POPropertie column : properties) {
            sql.append(" and " + column.getFieldNameByDB() + "=#{" + column.getFieldName() + "}");
        }

        sql.append("</where>");
        return sql.toString();
    }

//    public String where() {
//        StringBuilder sql = new StringBuilder();
//        sql.append(" where ");
//        List<POPropertie> properties = po.getFieldList();
//
//        List<String> list = new ArrayList<>();
//        for (POPropertie column : properties) {
//            list.add(column.getFieldNameByDB() + "=#{" + column.getFieldName() + "}");
//        }
//        sql.append(StringUtils.join(list.toArray()," and "));
//        return sql.toString();
//    }

    public StringBuilder whereByParam(Collection<String> param) {
        StringBuilder sql = new StringBuilder();
        if (param.size() > 0) {
            sql.append(" where ");
        }

        List<String> list = new ArrayList<>();
        for (String column : param) {
            POPropertie info = poUtil.getPropertieByName(column);
            list.add(info.getFieldNameByDB() + "=#{" + column + "}");
        }
        sql.append(StringUtils.join(list.toArray(), " and "));
        return sql;
    }

    /**
     * 不支持查询字符串为空的数据，若要查询，请使用wrapper
     *
     * @param column
     * @param contents
     * @return
     */
    private String ifNotNullXML(POPropertie column, String contents) {
        StringBuilder sql = new StringBuilder();
        sql.append(" <if test=\"");

        sql.append(column.getFieldName()).append(" != null");
        if (column.getJavaType().equals(String.class)) {//若是字符串，则验证是否为""
            sql.append(" and ");
            sql.append(column.getFieldName()).append(" != '' ");
        }
        sql.append("\">");
        sql.append(contents);
        sql.append("</if>");
        return sql.toString();
    }

    public StringBuilder orderByDefault() {
        StringBuilder sql = new StringBuilder();
        String orderBy = getOrderByStr();
        if (orderBy.length() == 0) {
            return sql;
        }

        sql.append(" order by ");
        sql.append(orderBy);
        return sql;
    }

    public String updateSetNotNullXML() {
        StringBuilder sql = new StringBuilder();
        sql.append("update ").append(po.getTableNameByDB()).append(" <set>");

        poUtil.getPropertieList().forEach(item -> {
            if (!item.isPrimarykey()) {
                sql.append(ifNotNullXML(item, item.getFieldNameByDB() + "=" + fieldDBWrite(item) + ","));
            }
        });

        sql.append("</set>");

        return sql.toString();
    }

    public StringBuilder updateSet(Collection<String> paramUpdateSet) {
        StringBuilder sql = new StringBuilder();
        List<String> list = new ArrayList<>();
        sql.append("update ").append(po.getTableNameByDB()).append(" set ");

        paramUpdateSet.forEach(
                item -> {
                    POPropertie info = poUtil.getPropertieByName(item);
                    list.add(info.getFieldNameByDB() + "=" + fieldDBWrite(info));
                }
        );
        sql.append(StringUtils.join(list.toArray(), ","));
        return sql;
    }



    public StringBuilder insert(Collection<String> paramUpdateSet) {
        List<String> fields = new ArrayList<>();
        List<String> values = new ArrayList<>();

        StringBuilder sql = new StringBuilder();
        sql.append("insert into ").append(po.getTableNameByDB()).append("(");
        paramUpdateSet.forEach(
                item -> {
                    POPropertie info = poUtil.getPropertieByName(item);
                    fields.add(info.getFieldNameByDB());
                }
        );
        sql.append(StringUtils.join(fields.toArray(), ","));
        sql.append(") values( ");

        paramUpdateSet.forEach(
                item -> {
                    POPropertie info = poUtil.getPropertieByName(item);
                    values.add(fieldDBWrite(info));
                }
        );
        sql.append(StringUtils.join(values.toArray(), ","));
        sql.append(")");

        return sql;
    }

    public StringBuilder deleteFrom() {
        StringBuilder sql = new StringBuilder();
        sql.append("delete from ");
        sql.append(po.getTableNameByDBAll());
        return sql;
    }

    public String insertNotNullXML() {
        StringBuilder sql = new StringBuilder();
        sql.append("insert into ").append(po.getTableNameByDB()).append(" (");
        sql.append("<trim suffixOverrides=\",\">");
        poUtil.getPropertieList().forEach(item -> {
            if (!item.isPrimarykey()) {
                sql.append(ifNotNullXML(item, item.getFieldNameByDB() + ","));
            }
        });
        sql.append("</trim>");

        sql.append(") values (");
        sql.append("<trim suffixOverrides=\",\">");

        poUtil.getPropertieList().forEach(item -> {
            if (!item.isPrimarykey()) {
                sql.append(ifNotNullXML(item, fieldDBWrite(item) + ","));
            }
        });
        sql.append("</trim>");

        sql.append(")");

        return sql.toString();
    }

    /**
     * 写入字段处理
     *
     * @param item
     * @return
     */
    private String fieldDBWrite(POPropertie item) {
        String name = "#{" + item.getFieldName() + "}";
        if (!StringUtils.isEmpty(item.getDbType())) {
            if ("datetime".equals(item.getDbType())) {
                name = "from_unixtime(#{" + item.getFieldName() + "})";
            }
        }
        return name;
    }

    /**
     * 读取时字段处理
     *
     * @param item
     * @return
     */
    private String fieldDBSelect(POPropertie item) {
        String name = item.getFieldNameByDB();
        if (!StringUtils.isEmpty(item.getDbType())) {
            if ("datetime".equals(item.getDbType())) {
                name = "unix_timestamp(" + item.getFieldNameByDB() + ") as " + item.getFieldName();
            }
        } else {
            if (!item.getFieldName().equals(item.getFieldNameByDB())) {
                name = item.getFieldNameByDB() + " as " + item.getFieldName();
            }
        }

        return name;
    }

    public String limit(int limit) {
        return " limit " + limit;
    }

    public String limit(int startIndex, int limit) {
        return " limit " + startIndex + "," + limit;
    }
}
