package com.qmrz.mybatis.service;

import com.qmrz.mybatis.sql.BaseWrapper;
import com.qmrz.mybatis.sql.sqlnode.UpdateParam;

import java.util.List;
import java.util.function.Consumer;

public interface BaseService<T> {
    List<T> selectWrapper(Consumer<? super BaseWrapper> wrapper);

    T selectOneWrapper(Consumer<? super BaseWrapper> wrapper);

    Integer updateWrapper(Object param, Consumer<? super BaseWrapper> wrapper);

    Integer deleteWrapper(Consumer<? super BaseWrapper> wrapper);

    /**
     * 根据实体中的属性值进行查询，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    List<T> select(Object whereObject);

    /**
     * 根据实体中的属性进行查询，只能有一个返回值，有多个结果取第1个
     *
     * @param whereObject map或实体
     * @return
     */
    T selectOne(Object whereObject);

    /**
     * 根据实体中的属性查询总数，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    Integer selectCount(Object whereObject);

    /**
     * 查询全部结果
     *
     * @return
     */
    List<T> selectAll();

    /**
     * 根据主键字段进行查询，查询条件使用等号
     *
     * @param paramPrimaryKey map或实体
     * @return
     */
    T selectByPrimaryKey(Object paramPrimaryKey);

    /**
     * 根据主键字段进行删除，方法参数必须包含完整的主键属性
     *
     * @param paramPrimaryKey map或实体
     * @return
     */
    int deleteByPrimaryKey(Object paramPrimaryKey);

    /**
     * 根据实体属性作为条件进行删除，查询条件使用等号
     *
     * @param whereObject map或实体
     * @return
     */
    int delete(Object whereObject);

    /**
     * 保存一个实体，null的属性不会保存，会使用数据库默认值
     *
     * @param entity map或实体
     * @return
     */
    int insert(T entity);

    /**
     * 根据主键更新实体全部字段，null值不会被更新
     *
     * @param paramMap map或实体
     * @return
     */
    int updateByPrimaryKey(Object paramMap);

    int update(UpdateParam updateParam);
}
