package com.qmrz.mybatis.sql;

import com.qmrz.mybatis.SqlUtil;

import java.util.List;
import java.util.Map;

/**
 * 基础条件接口
 */
public abstract class AbsWrapperData {

    /**
     * 0：where 1：select 2：from 3：order 5:group 6:limit
     * 20: update set
     * 30: insert
     * 40: delete
     */
    private int wrapperStatus = 0;

    /**
     * 当前块里的每个sql片段入栈时的条件组合方式
     * 0：无 1：and块 2：or块
     */
    private int conditionWhereStatus = 1;

    /**
     * 当前块的条件组合方式 0：无 1：and块 2：or块
     */
    private int wrapperWhereStatus = 1;

    public int getWrapperStatus() {
        return wrapperStatus;
    }

    public int getConditionWhereStatus() {
        return conditionWhereStatus;
    }

    public int getWrapperWhereStatus() {
        return wrapperWhereStatus;
    }

    public abstract StringBuilder getSQL();

    public StringBuilder getSQLLimit(){
        return new StringBuilder();
    }

    public abstract Map<String, Object> getParam();

    private Class<?> classPO;
    protected SqlUtil sqlUtil;

    /**
     * 总序号
     */
    private int index = 0;

    /**
     * 当前嵌套块的序号
     */
    private int currentIndex = 0;

    protected void setIndex(int index) {
        this.index = index;
        renameByParamName();
    }

    protected int getIndex() {
        return index;
    }

    protected void setCurrentIndex(int currentIndex) {
        this.currentIndex = currentIndex;
    }

    public AbsWrapperData(int wrapperStatus, int wrapperWhereStatus, int conditionWhereStatus, Class<?> classPO) {
        this.wrapperStatus = wrapperStatus;
        this.wrapperWhereStatus = wrapperWhereStatus;
        this.conditionWhereStatus = conditionWhereStatus;
        this.classPO = classPO;
        this.sqlUtil = new SqlUtil(classPO);
    }

    protected Class<?> getClassPO() {
        return classPO;
    }

    protected abstract void renameByParamName();

    /**
     * 防止参数重复，重命名增加序号
     *
     * @param fieldValue
     */
    protected void renameByParamName(ConditionGeneralValue fieldValue) {
        fieldValue.setName(fieldValue.getName() + "_" + index);
    }

    protected void renameByParamName(List<ConditionGeneralValue> fieldValue) {
        fieldValue.forEach(item -> item.setName(item.getName() + "_" + index));
    }

    /**
     * 获取连接符
     */
    public String getLinkStr() {
        String linkStr = " ";//每段sql的连接符，默认使用空格连接
        if (0 == currentIndex) {
            return linkStr;
        }

        if (0 == this.getWrapperStatus()) {//若where操作
            Integer whereStatus;
            //是否是块包装器
            if (this instanceof BaseWrapper) {
                whereStatus = this.getWrapperWhereStatus();
            } else {
                whereStatus = this.getConditionWhereStatus();
            }

            if (1 == whereStatus) {//若and
                linkStr = " and ";
            } else if (2 == whereStatus) {//若or
                linkStr = " or ";
            }
        }
        return linkStr;
    }
}
