package com.qmrz.mybatis.sql;

import com.qmrz.mybatis.sql.statement.*;

import java.util.*;
import java.util.function.Consumer;

public abstract class BaseWrapper extends AbsWrapperData implements IWrapper {
    /**
     * where 连接状态
     * 1: and块
     * 2: or块
     */
    //protected int whereStatus = 1;
    private ConditionCount conditionsCount = new ConditionCount(0);
    private int conditionsCurrentCount = 0;
    private List<AbsWrapperData> conditions = new ArrayList<>();

    protected void setConditionsCount(ConditionCount conditionsCount) {
        this.conditionsCount = conditionsCount;
    }

    public void addStatement(AbsWrapperData absWrapperData) {
        absWrapperData.setIndex(conditionsCount.getCount());
        absWrapperData.setCurrentIndex(conditionsCurrentCount);
        conditions.add(absWrapperData);
        conditionsCount.increment();
        conditionsCurrentCount++;
    }

    public List<AbsWrapperData> getConditions() {
        return conditions;
    }

    public BaseWrapper(int wrapperStatus, int wrapperWhereStatus, int conditionWhereStatus, Class<?> classPO) {
        super(wrapperStatus, wrapperWhereStatus, conditionWhereStatus, classPO);
//        Class<?> classPO = (Class<?>) ((ParameterizedType) (this.getClass().getGenericInterfaces()[0])).getActualTypeArguments()[0];
//        POUtil.getPropertie(classPO);
    }

    /**
     * 此块的完整sql
     *
     * @return
     */
    @Override
    public StringBuilder getSQL() {
        StringBuilder sql = new StringBuilder();
        if (conditions.size() == 0) {
            return sql;
        }
        boolean kuohao = 0 == this.getWrapperStatus() && 0 != this.getWrapperWhereStatus();
        if (kuohao) {//where的条件时
            sql.append(this.getLinkStr()).append("(");
        }

        conditions.forEach(item -> {
            if (6 != item.getWrapperStatus()) {//6 limit
                sql.append(item.getSQL());
            }
        });

        if (kuohao) {//where时
            sql.append(")");
        }
        return sql;
    }

    @Override
    public StringBuilder getSQLLimit() {
        StringBuilder sql = new StringBuilder();
        if (conditions.size() == 0) {
            return sql;
        }

        conditions.forEach(item -> {
//            System.out.println("getSQLLimit");
            sql.append(item.getSQLLimit());
        });

        return sql;
    }

    /**
     * 此块的所有参数
     *
     * @return
     */
    public Map<String, Object> getParam() {
        Map<String, Object> param = new LinkedHashMap<>();
        conditions.forEach(item -> {
            if (item.getParam() != null) {
                for (Map.Entry<String, Object> entry : item.getParam().entrySet()) {
                    param.put(entry.getKey(), entry.getValue());
                }
            }
        });
        return param;
    }

    @Override
    protected void renameByParamName() {
        //块包装器只是块，无参数，无逻辑
    }

    public BaseWrapper selectAllField() {
        addStatement(new SelectAllField(getClassPO()));
        return this;
    }

    public BaseWrapper update(Map<String, Object> setParam) {
        addStatement(new UpdateSet(getClassPO(), setParam));
        return this;
    }

    public BaseWrapper delete() {
        addStatement(new DeleteFrom(getClassPO()));
        return this;
    }

    public BaseWrapper from() {
        addStatement(new FromTable(getClassPO()));
        return this;
    }

    public BaseWrapper limit(int limit) {
        addStatement(new Limit(getClassPO(), limit, null));
        return this;
    }

    public BaseWrapper limit(int limit1, int limit2) {
        addStatement(new Limit(getClassPO(), limit1, limit2));
        return this;
    }

    public BaseWrapper between(String fieldName, Object fieldValue1, Object fieldValue2) {
        List<ConditionGeneralValue> param = new ArrayList<>();
        param.add(new ConditionGeneralValue(fieldName + "_between1", fieldValue1));
        param.add(new ConditionGeneralValue(fieldName + "_between2", fieldValue2));

        ConditionBetween c = new ConditionBetween(getWrapperWhereStatus()
                , getConditionWhereStatus(), getClassPO());
        c.setFiedName(fieldName);
        c.setFieldValue(param);
        return addWhere(c);
    }

    public BaseWrapper in(String fieldName, List value) {
        return in(fieldName, value.toArray());
    }

    public BaseWrapper in(String fieldName, Object ...value) {
        if (value == null || value.length==0) {
            return this;
        }
        List<ConditionGeneralValue> param = new ArrayList<>();
        for (int i=0;i<value.length;i++) {
            String name=fieldName+"_in_"+String.valueOf(i);
            ConditionGeneralValue val=new ConditionGeneralValue(name,value[i]);
            param.add(val);
        }

        ConditionIn c = new ConditionIn(getWrapperWhereStatus()
                , getConditionWhereStatus(), getClassPO());
        c.setFiedName(fieldName);
        c.setFieldValue(param);
        return addWhere(c);
    }

    /**
     * @param f     字段名称
     * @param value
     * @return
     */
    public BaseWrapper eq(String f, Object value) {
        return addWhere(f, "_eq", value, "=");
    }

    public BaseWrapper neq(String fieldName, Object value) {
        return addWhere(fieldName, "_neq", value, "<>");
    }

    public BaseWrapper gt(String fieldName, Object value) {
        return addWhere(fieldName, "_gt", value, ">");
    }

    public BaseWrapper ge(String fieldName, Object value) {
        return addWhere(fieldName, "_ge", value, ">=");
    }

    public BaseWrapper lt(String fieldName, Object value) {
        return addWhere(fieldName, "_lt", value, "<");
    }

    public BaseWrapper le(String fieldName, Object value) {
        return addWhere(fieldName, "_le", value, "<=");
    }

    public BaseWrapper or(Consumer<? super BaseWrapper> action) {
        return or(true, action);
    }

    public BaseWrapper orderByDefault() {
        addStatement(new OrderByDefault(getClassPO()));
        return this;
    }

    public BaseWrapper like(String fieldName, String fieldValue) {
        if (fieldValue == null) {
            return this;
        }

        ConditionLike c = new ConditionLike(this.getWrapperWhereStatus()
                , this.getConditionWhereStatus(), getClassPO());

        c.setFiedName(fieldName);
        c.setFieldValue(new ConditionGeneralValue(fieldName + "_like", fieldValue));
        return addWhere(c);
    }

    public BaseWrapper or(boolean isConditionAnd, Consumer<? super BaseWrapper> action) {
        OrBlock w = new OrBlock(isConditionAnd, this.conditionsCount, this.getClassPO());
        this.addStatement(w);
        action.accept(w);//执行嵌套的链式调用
        return this;
    }


    public BaseWrapper where() {
        this.addStatement(
                new StrWhere(this.getWrapperWhereStatus(),
                        this.getConditionWhereStatus()
                        , getClassPO())
        );
        return this;
    }

    public BaseWrapper and(Consumer<? super BaseWrapper> action) {
        return and(true, action);
    }

    public BaseWrapper and(boolean isConditionAnd, Consumer<? super BaseWrapper> action) {
        AndBlock w = new AndBlock(isConditionAnd, this.conditionsCount, this.getClassPO());
        this.addStatement(w);
        action.accept(w);//执行嵌套的链式调用
        return this;
    }

    /**
     * 添加where语句
     *
     * @param fieldName   参数名
     * @param paramSuffix 参数后缀
     * @param fieldValue  参数值
     * @param condition   操作符 = > <
     * @return
     */
    private BaseWrapper addWhere(String fieldName, String paramSuffix, Object fieldValue, String condition) {
        ConditionGeneral c = new ConditionGeneral(this.getWrapperWhereStatus()
                , this.getConditionWhereStatus(), getClassPO());

        c.setFiedName(fieldName);
        c.setFieldValue(new ConditionGeneralValue(fieldName + paramSuffix, fieldValue));
        c.setCondition(condition);
        return addWhere(c);
    }

    /**
     * 添加where 条件语句
     *
     * @param c
     * @return
     */
    private BaseWrapper addWhere(AbsWrapperData c) {
        int conditionWhere = this.getConditionWhereStatus();

        //若只第一次where，则增加一个where包装，使状态conditionWhere进入and/or状态
        if (conditionWhere != 1 && conditionWhere != 2) {//默认为and
            WhereBlock ab = new WhereBlock(true, conditionsCount, getClassPO());
            ab.where().addStatement(c);
            this.addStatement(ab);
            return ab;
        }

        this.addStatement(c);
        return this;
    }

    public BaseWrapper notoption() {
        return this;
    }
}
