package com.qmrz.utils;

import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.executor.BatchResult;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

/**
 * 基于mybatis的数据库操作类，无需mapper接口，直接调用xml的sql
 */
@Component
public class DBHelper implements SqlSession {
    SqlSessionTemplate sql;
    private static final String statementPrefix = "com.qmrz.domain.mapper.";

    public String getStatement(String id) {
        return statementPrefix + id;
    }

    @Autowired
    public DBHelper(SqlSessionTemplate sqlSessionTemplate) {
        this.sql = sqlSessionTemplate;
    }

    @Override
    public <T> T selectOne(String statement) {
        return sql.selectOne(getStatement(statement));
    }

    @Override
    public <T> T selectOne(String statement, Object parameter) {
        return sql.selectOne(getStatement(statement), parameter);
    }

    @Override
    public <E> List<E> selectList(String statement) {
        return sql.selectList(getStatement(statement));
    }

    @Override
    public <E> List<E> selectList(String statement, Object parameter) {
        return sql.selectList(getStatement(statement), parameter);
    }

    @Override
    public <E> List<E> selectList(String statement, Object parameter, RowBounds rowBounds) {
        return sql.selectList(getStatement(statement), parameter, rowBounds);
    }

    @Override
    public <K, V> Map<K, V> selectMap(String statement, String mapKey) {
        return sql.selectMap(getStatement(statement), mapKey);
    }

    @Override
    public <K, V> Map<K, V> selectMap(String statement, Object parameter, String mapKey) {
        return sql.selectMap(getStatement(statement), parameter, mapKey);
    }

    @Override
    public <K, V> Map<K, V> selectMap(String statement, Object parameter, String mapKey, RowBounds rowBounds) {
        return sql.selectMap(getStatement(statement), parameter, mapKey, rowBounds);
    }

    @Override
    public <T> Cursor<T> selectCursor(String statement) {
        return sql.selectCursor(getStatement(statement));
    }

    @Override
    public <T> Cursor<T> selectCursor(String statement, Object parameter) {
        return sql.selectCursor(getStatement(statement), parameter);
    }

    @Override
    public <T> Cursor<T> selectCursor(String statement, Object parameter, RowBounds rowBounds) {
        return sql.selectCursor(getStatement(statement), parameter, rowBounds);
    }

    @Override
    public void select(String statement, Object parameter, ResultHandler handler) {
        sql.select(getStatement(statement), parameter, handler);
    }

    @Override
    public void select(String statement, ResultHandler handler) {
        sql.select(getStatement(statement), handler);
    }

    @Override
    public void select(String statement, Object parameter, RowBounds rowBounds, ResultHandler handler) {
        sql.select(getStatement(statement), parameter, rowBounds, handler);
    }

    @Override
    public int insert(String statement) {
        return sql.insert(getStatement(statement));
    }

    @Override
    public int insert(String statement, Object parameter) {
        return sql.insert(getStatement(statement), parameter);
    }

    @Override
    public int update(String statement) {
        return sql.update(getStatement(statement));
    }

    @Override
    public int update(String statement, Object parameter) {
        return sql.update(getStatement(statement), parameter);
    }

    @Override
    public int delete(String statement) {
        return sql.delete(getStatement(statement));
    }

    @Override
    public int delete(String statement, Object parameter) {
        return sql.delete(getStatement(statement), parameter);
    }

    @Override
    public void commit() {
        sql.commit();
    }

    @Override
    public void commit(boolean force) {
        sql.commit(force);
    }

    @Override
    public void rollback() {
        sql.rollback();
    }

    @Override
    public void rollback(boolean force) {
        sql.rollback(force);
    }

    @Override
    public List<BatchResult> flushStatements() {
        return sql.flushStatements();
    }

    @Override
    public void close() {
        sql.close();
    }

    @Override
    public void clearCache() {
        sql.clearCache();
    }

    @Override
    public Configuration getConfiguration() {
        return sql.getConfiguration();
    }

    @Override
    public <T> T getMapper(Class<T> type) {
        return sql.getMapper(type);
    }

    @Override
    public Connection getConnection() {
        return sql.getConnection();
    }

}
