package com.qmrz.utils;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAccessor;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

public class DateTimeUtil {
    private static final DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private static final DateTimeFormatter dtfDate = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    private static ThreadLocal<SimpleDateFormat> ThreadDateTime = new ThreadLocal<SimpleDateFormat>();

    //日期时间类型格式
    private static final String DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";

    private static final DateTimeFormatter dtfGMT = DateTimeFormatter.ofPattern("EEE, dd MMM yyyy HH:mm:ss z", Locale.ENGLISH)
            .withZone(TimeZone.getTimeZone("GMT").toZoneId());

    public static String format(TemporalAccessor dateTime) {
        return dtf.format(dateTime);
    }

    public static String format(TemporalAccessor dateTime, String format) {
        return DateTimeFormatter.ofPattern(format).format(dateTime);
    }

    public static String formatDate(TemporalAccessor dateTime) {
        return dtfDate.format(dateTime);
    }

    public static String datePlus(String date,int days){
        //结束日期需要加一天，api同步时不包含结束日期
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime local = LocalDateTime.from(dtf.parse(date+" 00:00:00")).plusDays(days);
        return formatDate(local);
    }

//    public static String formatBySecond(long second) {
//        return dtf.format(LocalDateTime.ofEpochSecond(second, 0, ZoneOffset.ofHours(8)));
//    }
//
//    public static String formatDataBySecond(long second) {
//        return dtfDate.format(LocalDateTime.ofEpochSecond(second, 0, ZoneOffset.ofHours(8)));
//    }

    public static String formatGMTBySecond(long second) {
        return dtfGMT.format(LocalDateTime.ofEpochSecond(second, 0
                , ZoneOffset.ofHours(0)));
    }

    /**
     * 获取当前日期 毫秒
     *
     * @return
     */
    public static long getTimeInMillis() {
        Calendar now = Calendar.getInstance();

        return now.getTimeInMillis();
    }

    /**
     * 时间字符串 解析为 时间戳（毫秒）
     *
     * @param time   时间，如：2018-08-06 04:52:50 127
     * @param format 时间格式，如：yyyy-MM-dd HH:mm:ss SSS
     * @param zone   时区，如：ZoneOffset.ofHours(8)
     * @return
     */
    public static Long getTimeInMillis(String time, String format, ZoneOffset zone) {
        Instant instant = getInstant(time, format, zone);
        return instant.toEpochMilli();
    }

    /**
     * 获取当前日期 秒
     *
     * @return
     */
    public static long getTimeInSeconds() {
        return getTimeInMillis() / 1000L;
    }

    /**
     * 时间字符串 解析为 时间戳（秒）
     *
     * @param time   时间，如：2018-08-06 04:52:50 127
     * @param format 时间格式，如：yyyy-MM-dd HH:mm:ss SSS
     * @param zone   时区，如：ZoneOffset.ofHours(8)
     * @return
     */
    public static Long getTimeInSeconds(String time, String format, ZoneOffset zone) {
        Instant instant = getInstant(time, format, zone);
        return instant.getEpochSecond();
    }

    /**
     * 时间字符串 解析为 时刻对象
     *
     * @param time
     * @param format
     * @param zone
     * @return
     */
    public static Instant getInstant(String time, String format, ZoneOffset zone) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern(format);
        Instant instant = LocalDateTime.parse(time, df).atZone(zone).toInstant();
        return instant;
    }

    /**
     * 获取时刻 时间戳 转 时刻
     *
     * @param timestamp
     * @return
     */
    public static Instant getInstant(long timestamp) {
        return Instant.ofEpochSecond(timestamp);
    }

    /**
     * 获取当前日期时间
     *
     * @return 返回当前时间的字符串值
     */
    public static String currentDateTime() {
        return DateTimeInstance().format(new Date());
    }

    private static SimpleDateFormat DateTimeInstance() {
        SimpleDateFormat df = ThreadDateTime.get();
        if (df == null) {
            df = new SimpleDateFormat(DATETIME_FORMAT);
            ThreadDateTime.set(df);
        }
        return df;
    }

}
