package com.qmrz.utils;

import com.alibaba.fastjson.JSONObject;
import com.qmrz.utils.classload.JavaStringCompiler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.DigestUtils;

import java.io.*;
import java.net.URLEncoder;
import java.util.*;

@Slf4j
public class Fn {
    public static String md5(String s) {
        s = "af3zs" + s + "1678";
        byte[] arr = new byte[0];
        try {
            arr = DigestUtils.md5Digest(s.getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return DigestUtils.md5DigestAsHex(arr);
    }

    /**
     * 对象转int(默认是对象的toString())，失败返回 failureVal
     *
     * @param o
     * @param failureVal
     * @return
     */
    public static Integer toInt(Object o, int failureVal) {
        if (o == null) {
            return failureVal;
        }
        String str = o.toString();

        try {
            return Integer.parseInt(str);
        } catch (NumberFormatException ex) {
            return failureVal;
        }
    }

    /**
     * 对象转int(默认是对象的toString())，失败返回 0
     *
     * @param o
     * @return
     */
    public static Integer toInt(Object o) {
        if (o == null) {
            return 0;
        }
        String str = o.toString();
        try {
            return Integer.parseInt(str);
        } catch (NumberFormatException ex) {
            return 0;
        }
    }

    public static boolean isInt(Object o) {
        if (o == null) {
            return false;
        }
        try {
            Integer.parseInt(o.toString());
            return true;
        } catch (NumberFormatException ex) {
            return false;
        }
    }

    public static Long toLong(Object o) {
        if (o == null) {
            return 0l;
        }
        String str = o.toString();
        try {
            return Long.parseLong(str);
        } catch (NumberFormatException ex) {
            return 0l;
        }
    }

    public static boolean isDouble(Object o) {
        if (o == null) {
            return false;
        }
        try {
            Double.parseDouble(o.toString());
            return true;
        } catch (NumberFormatException ex) {
            return false;
        }
    }

    public static Integer toInt16(Object o) {
        if (o == null) {
            return 0;
        }
        try {
            return Integer.parseInt(o.toString(), 16);
        } catch (NumberFormatException ex) {
            return 0;
        }
    }

    public static boolean isInt16(Object o) {
        if (o == null) {
            return false;
        }
        try {
            Integer.parseInt(o.toString(), 16);
            return true;
        } catch (NumberFormatException ex) {
            return false;
        }
    }

    public static String toString(Object o){
        if (o == null) {
            return "";
        }
        return o.toString();
    }

    public static String toString(String str){
        if (str == null) {
            return "";
        }
        return str;
    }

    /**
     * 去除字符串前后指定字符
     *
     * @param value
     * @param cha
     * @return
     */
    public static String trim(String value, char cha) {
        int len = value.length();

        int st = 0;
        char[] val = value.toCharArray();

        while ((st < len) && (val[st] <= cha)) {
            st++;
        }
        while ((st < len) && (val[len - 1] <= cha)) {
            len--;
        }
        return ((st > 0) || (len < value.length())) ? value.substring(st, len) : value;
    }

    /**
     * 是否来自ajax请求
     *
     * @param requestContent
     * @return
     */
    public static Boolean requestIsAjax(String requestContent) {
        if (requestContent == null || requestContent.length() == 0) {
            return false;
        }
        JSONObject o = null;
        try {
            o = Json.toObject(requestContent);
        } catch (Throwable e) {
            return false;
        }

        if (!o.containsKey("ajax")) {
            return false;
        }
        return new Integer("1").equals(o.get("ajax"));
    }

    /**
     * 是否来自ajax请求
     *
     * @param inputStream
     * @return
     */
    public static Boolean requestIsAjax(InputStream inputStream) {
        String input = StreamUtil.inputToString(inputStream).toString();
        return requestIsAjax(input);
    }

    public static String urlEnodeUTF8(String str) {
        String result = str;
        try {
            result = URLEncoder.encode(str, "UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取一定长度的随机字符串
     *
     * @param length 指定字符串长度
     * @return 一定长度的字符串
     */
    public static String getRandomString(int length) {
        String base = "abcdefghijklmnopqrstuvwxyz0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    /**
     * 生成指定位数数字
     *
     * @param length
     * @return
     */
    public static String getRandomNumber(int length) {
        // 生成随机n位码
        String s = "";
        while (s.length() < 4) {
            s += (int) (Math.random() * 10);
        }
        return s;
    }

    /**
     * 生成唯一字符串，去除横杆
     *
     * @return
     */
    public static String getUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 对象序列化为字符串
     *
     * @return
     */
    public static String objectSerialize(Object obj) {
        ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
        try {
            ObjectOutputStream outputStream = new ObjectOutputStream(byteOutput);
            outputStream.writeObject(obj);
            String str = byteOutput.toString("ISO-8859-1");

            return str;
        } catch (IOException e) {
            e.printStackTrace();
            return "error:" + e.getMessage();
        }
    }

    /**
     * 字符串反序列化为对象
     *
     * @param str
     * @return
     */
    public static Object objectDeserialize(String str) {
        try {
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(str.getBytes("ISO-8859-1"));
            ObjectInputStream objectInputStream = new ObjectInputStream(byteArrayInputStream);
            Object obj = objectInputStream.readObject();
            return obj;
        } catch (Exception e) {
            e.printStackTrace();
            return "error:" + e.getMessage();
        }
    }

    /**
     * 首字母大写
     *
     * @param str
     * @return
     */
    public static String firstUpperCase(String str) {
        if (str == null) {
            return null;
        }
        if (0 == str.length()) {
            return "";
        }

        return str.substring(0, 1).toUpperCase() + str.substring(1);
    }

    /**
     * 首字母大写（高性能，待验证）
     *
     * @param str
     * @return
     */
    public static String firstUpperCase2(String str) {
        if (str == null) {
            return null;
        }
        if (0 == str.length()) {
            return "";
        }

        char[] arr = str.toCharArray();
        arr[0] -= 32;
        return String.valueOf(arr);
    }

    /**
     * 手动metaspace扩容，以便即时触发，释放OC内存（非回收）
     *
     * @param cou         扩容数量
     * @param sleepSecond sleepSecond 秒后执行
     */
    public static void addMetaspace(int cou, long sleepSecond) {
        log.info("准备metaspace扩容：" + cou + "个");
        try {
            //5秒后发生
            Thread.sleep(sleepSecond * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        StringBuffer sb = new StringBuffer();
        sb.append("package com.qmrz.utils.classload;\n");
        sb.append("public class MetaspaceEmptyCapacityExpansion");

        StringBuffer sb2 = new StringBuffer();
        sb2.append(" {\n");
        sb2.append("private static String str = String.valueOf(System.currentTimeMillis());\n");
        sb2.append("public void test(){}\n");
        for (int i = 0; i < 1000; i++) {
            sb2.append("public void test" + String.valueOf(i) + "(){}\n");
        }
        sb2.append("}");

        List<Class<?>> claList = new ArrayList<>();
        JavaStringCompiler compiler = new JavaStringCompiler();
        for (int i = 0; i < cou; i++) {
            String strI = String.valueOf(i);
            StringBuffer tmp = new StringBuffer();
            tmp.append(sb);
            tmp.append(strI);
            tmp.append(sb2);

            Map<String, byte[]> results = null;
            try {
                results = compiler.compile("MetaspaceEmptyCapacityExpansion" + strI + ".java", tmp.toString());
            } catch (IOException e) {
                e.printStackTrace();
            }

            try {
                Class<?> clazz = compiler.loadClass("com.qmrz.utils.classload.MetaspaceEmptyCapacityExpansion" + strI, results);
                claList.add(clazz);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            tmp = null;
            log.info("忽略警告 " + i);
        }

        claList.clear();
        claList = null;
    }
}
