package com.qmrz.utils;

import lombok.ToString;

/**
 * 统一数据响应格式
 * json格式：{"status":10200,"msg":"success","data":任何数据}
 */
@ToString
public class RD {
    private int status;
    private String msg;
    private Object data;

    public RD(RDCode status, String msg, Object data) {
        this.status = status.getState();
        this.msg = msg;
        this.data = data;
    }

    public static RD create(RDCode state, String msg, Object data) {
        return new RD(state, msg, data);
    }

    public static RD create(RDCode state,Object data) {
        return new RD(state, state.getMessage(), data);
    }

    public static RD create(RDCode state) {
        return new RD(state, state.getMessage(), "");
    }

    public static RD notfound() {
        return create(RDCode.notfound, "");
    }

    public static RD notlogin() {
        return create(RDCode.notlogin, "");
    }

    public static RD usernotlogin(){
        return create(RDCode.usernotlogin,  "");
    }

    public static RD exception(String msg) {
        return create(RDCode.exception, msg, "");
    }

    public static RD notpermission() {
        return create(RDCode.nopermission, "");
    }

    //失败
    public static RD failure() {
        return create(RDCode.failure, "");
    }

    public static RD failure(String msg) {
        return create(RDCode.failure, msg, "");
    }

    public static RD failure(String msg, Object data) {
        return create(RDCode.failure, msg, data);
    }

    public static RD failureByForm(String msg, Object data) {
        return create(RDCode.failureByForm, msg, data);
    }

    //成功
    public static RD success() {
        return create(RDCode.success, "");
    }

    public static RD success(Object data) {
        return create(RDCode.success,  data);
    }

    public static RD success(String msg, Object data) {
        return create(RDCode.success, msg, data);
    }

    //快速
    public static RD quick(boolean b) {
        if (b) {
            return success();
        } else {
            return failure();
        }
    }

    /**
     * 成功：返回数据，失败：返回失败信息
     *
     * @param b
     * @param failureMsg  失败时返回失败信息
     * @param successdata 成功时返回数据
     * @return
     */
    public static RD quick(boolean b, String failureMsg, Object successdata) {
        if (b) {
            return success(successdata);
        } else {
            return failure(failureMsg);
        }
    }

    public int getStatus() {
        return status;
    }

    public String getMsg() {
        return msg;
    }

    public Object getData() {
        return data;
    }
}
