package com.qmrz.utils;

import com.qmrz.exception.ABException;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.imageio.ImageIO;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.Calendar;

@Slf4j
public class Web {
    public static void redirect(String path) throws IOException {
        getResponse().sendRedirect(path);
    }

    public static void setNoCache() {
        getResponse().addHeader("Cache-Control", "no-cache");
    }

    public static void setCache() {
        HttpServletResponse response = getResponse();
        response.setHeader("Cache-Control", "max-age=315360000");

        Calendar now = Calendar.getInstance();
        now.add(Calendar.YEAR, 10);
        response.setHeader("Expires", DateTimeUtil.formatGMTBySecond(now.getTimeInMillis() / 1000L));
    }

    public static HttpServletResponse getResponse() {
        return getRequestAttributes().getResponse();
    }

    public static HttpServletRequest getRequest() {
        return getRequestAttributes().getRequest();
    }

    public static ServletRequestAttributes getRequestAttributes() {
        return (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    }

    public static String getRequestBody() {
        String line = "";
        String result = null;
        HttpServletRequest request = Web.getRequest();
        try {
            BufferedReader reader = request.getReader();
            StringBuffer inputString = new StringBuffer();
            while ((line = reader.readLine()) != null) {
                inputString.append(line);
            }
            result = inputString.toString();
            request.getReader().close();
        } catch (IOException e) {
            String msg = "获取请求内容时异常：" + e.getMessage();
            log.info(msg);
            throw new ABException(msg);
        }
        return result;
    }

    public static void printRD(ServletResponse response, RD rd) {
        //response.setHeader("Cache-Control","no-cache");
        response.setContentType("application/json;charset=UTF-8");

        try {
            response.getWriter().print(JSONObject.toJSONString(rd));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void responseImage(String fileName) {
        Web.getResponse().setContentType("image/jpeg");

        File file = new File(fileName);

        FileInputStream inputStream = null;
        try {
            inputStream = new FileInputStream(file);
            byte[] data = new byte[(int) file.length()];
            int length = inputStream.read(data);
            inputStream.close();
            OutputStream stream = Web.getResponse().getOutputStream();
            stream.write(data);
            stream.flush();
            stream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * InputStream转数组
     * @param input
     * @return
     * @throws IOException
     */
    private static byte[] toByteArray(InputStream input) throws IOException {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int n = 0;
        while (-1 != (n = input.read(buffer))) {
            output.write(buffer, 0, n);
        }
        return output.toByteArray();
    }

    /**
     * 响应InputStream
     * @param inputStream
     */
    public static void responseInputStream(InputStream inputStream) {
        try {
            byte[] data = toByteArray(inputStream);
            inputStream.close();
            OutputStream stream = Web.getResponse().getOutputStream();
            stream.write(data);
            stream.flush();
            stream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void responseImage2(String fileName) {
        HttpServletResponse response = Web.getResponse();
        response.setContentType("image/jpeg");
        response.setHeader("Cache-Control", "max-age=315360000");

        Calendar now = Calendar.getInstance();
        now.add(Calendar.YEAR, 10);
        response.setHeader("Expires", DateTimeUtil.formatGMTBySecond(now.getTimeInMillis() / 1000L));

//        response.setHeader("Accept-Ranges", "bytes");
        Image image2 = Toolkit.getDefaultToolkit().getImage(fileName);
        log.info(fileName);
        try {
            BufferedImage image3 = ImageUtil.toBufferedImage(image2);
            ImageIO.write(image3, "jpg", Web.getResponse().getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

//    public static void responseImage2(InputStream inputStream,int streamLength) {
//        HttpServletResponse response = Web.getResponse();
//        response.setContentType("image/jpeg");
//        response.setHeader("Cache-Control", "max-age=315360000");
//
//        Calendar now = Calendar.getInstance();
//        now.add(Calendar.YEAR, 10);
//        response.setHeader("Expires", DateTimeUtil.formatGMTBySecond(now.getTimeInMillis() / 1000L));
//
////        response.setHeader("Accept-Ranges", "bytes");
//        Image image2 = Toolkit.getDefaultToolkit().getImage(fileName);
//        Toolkit.getDefaultToolkit().getImage()
//        System.out.println(fileName);
//        try {
//            BufferedImage image3 = ImageUtil.toBufferedImage(image2);
//            ImageIO.write(image3, "jpg", Web.getResponse().getOutputStream());
//        } catch (Exception e) {
//            log.error(e.getMessage());
//        }
//    }

    public static void responseText(String content) {
        if (content == null) {
            content = "";
        }
        Web.getResponse().setContentType("text/plain");
        PrintWriter writer = null;
        try {
            writer = Web.getResponse().getWriter();
            writer.write(content);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (writer != null) {
                writer.flush();
                writer.close();
            }
        }
    }

    public static void responseText(String content, String contentType) {
        if (content == null) {
            content = "";
        }
        Web.getResponse().setContentType(contentType);
        PrintWriter writer = null;
        try {
            writer = Web.getResponse().getWriter();
            writer.write(content);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (writer != null) {
                writer.flush();
                writer.close();
            }
        }
    }
}
