package com.qmrz.utils.coordinate;

import com.qmrz.exception.FailureException;
import com.qmrz.utils.Fn;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;

/**
 *
 */
@Slf4j
@Data
public class Angle {
    private double degree;
    private double minute;
    private double second;

    public double toDouble() {
        double d = degree + minute / 60 + second / 3600;
        BigDecimal b = new BigDecimal(d);
        return b.setScale(6, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static Angle parse(String s) {
        Angle angle = new Angle();
        char[] chars = s.toCharArray();
        String num = "";
        for (char ch : chars) {
            if (Character.isDigit(ch)) {
                num += ch;
            } else if (ch == '度' || ch == '°') {
                angle.setDegree(Integer.parseInt(num));
                num = "";
            } else if (ch == '分' || ch == '\'') {
                angle.setMinute(Integer.parseInt(num));
                num = "";
            } else if (ch == '秒' || ch == '\"') {
                angle.setSecond(Integer.parseInt(num));
                num = "";
            } else {
                throw new FailureException("错误的格式。");
            }
        }

        return angle;
    }

    public static boolean check(String s) {
        Angle angle = new Angle();
        char[] chars = s.toCharArray();
        String num = "";
        for (char ch : chars) {
            if (Character.isDigit(ch)) {
                num += ch;
            } else if (ch == '度' || ch == '°') {
                if (!Fn.isInt(num)) {
                    return false;
                }

                num = "";
            } else if (ch == '分' || ch == '\'') {
                if (!Fn.isInt(num)) {
                    return false;
                }
                num = "";
            } else if (ch == '秒' || ch == '\"') {
                if (!Fn.isInt(num)) {
                    return false;
                }
                num = "";
            } else {
                return false;
            }
        }

        return true;
    }

    public static void main(String[] args) {
        log.info(String.valueOf(check("13度23分89秒")));
        log.info(String.valueOf(check("13°23'89")));
        log.info(String.valueOf(Angle.parse("13度23分89秒")));
        Angle a = Angle.parse("13°23'89");
        log.info(String.valueOf(a));
        log.info(String.valueOf(a.toDouble()));
    }
}
