package com.qmrz.utils.http;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qmrz.utils.Json;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.Header;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
public class HttpUtil {
    public static JSONObject httpsRequestJson(String requestUrl, String requestMethod, String outputStr) {
        JSONObject jsonObject = null;
        try {
            TrustManager[] tm = {new MyTrustManager()};
            SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
            sslContext.init(null, tm, new java.security.SecureRandom());
            SSLSocketFactory ssf = sslContext.getSocketFactory();

            URL url = new URL(requestUrl);
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setSSLSocketFactory(ssf);

            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod(requestMethod);
            if (null != outputStr) {
                OutputStream outputStream = conn.getOutputStream();
                outputStream.write(outputStr.getBytes("UTF-8"));
                outputStream.close();
            }
            InputStream inputStream = conn.getInputStream();
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            String str = null;
            StringBuffer buffer = new StringBuffer();
            while ((str = bufferedReader.readLine()) != null) {
                buffer.append(str);
            }
            bufferedReader.close();
            inputStreamReader.close();
            inputStream.close();
            inputStream = null;
            conn.disconnect();
            log.info(" -- url: " + requestUrl);
//            log.info(" -- result: " + buffer.toString());
            jsonObject = JSONObject.parseObject(buffer.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return jsonObject;
    }

    public static Object httpRequestJson(String requestUrl, String requestMethod, String outputStr) {
        int responseCode = 30500;//默认系统异常
        try {
            URL url = new URL(requestUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();

            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod(requestMethod.toUpperCase());
            if (null != outputStr) {
                OutputStream outputStream = conn.getOutputStream();
                outputStream.write(outputStr.getBytes("UTF-8"));
                outputStream.close();
            }

            InputStream inputStream;
            responseCode = conn.getResponseCode();
            if (responseCode >= 400) {
                inputStream = conn.getErrorStream();
            } else {
                inputStream = conn.getInputStream();
            }

            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            String str = null;
            StringBuffer buffer = new StringBuffer();
            while ((str = bufferedReader.readLine()) != null) {
                buffer.append(str);
            }
            bufferedReader.close();
            inputStreamReader.close();
            inputStream.close();
            inputStream = null;
            conn.disconnect();
            log.info(" -- url: " + requestUrl);
//            log.info(" -- result: " + buffer.toString());
            Object jsonObject = Json.parse(buffer.toString());
            return jsonObject;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static String getRequestFullUri(HttpServletRequest request) {
        String port = "";
        if (request.getServerPort() != 80) {
            port = ":" + request.getServerPort();
        }
        return request.getScheme() + "://" + request.getServerName() + port + request.getContextPath() + request.getServletPath();
    }

    public static String httpsRequest(String requestUrl, String requestMethod, String outputStr) {
        String retStr = "";
        try {
            TrustManager[] tm = {new MyTrustManager()};
            SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
            sslContext.init(null, tm, new java.security.SecureRandom());
            SSLSocketFactory ssf = sslContext.getSocketFactory();

            URL url = new URL(requestUrl);
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setSSLSocketFactory(ssf);

            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod(requestMethod);
            if (null != outputStr) {
                OutputStream outputStream = conn.getOutputStream();
                outputStream.write(outputStr.getBytes("UTF-8"));
                outputStream.close();
            }
            InputStream inputStream = conn.getInputStream();
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            String str = null;
            StringBuffer buffer = new StringBuffer();
            while ((str = bufferedReader.readLine()) != null) {
                buffer.append(str);
            }
            bufferedReader.close();
            inputStreamReader.close();
            inputStream.close();
            conn.disconnect();
            log.info(" -- url: " + requestUrl);
//            log.info(" -- result: " + buffer.toString());
            retStr = buffer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return retStr;
    }

    public static HttpResult httpsRequestMap(String requestUrl, String requestMethod, String outputStr
            , Map<String, String> header) {
        int responseCode = 30500;//默认系统异常
        String body = "";
        String msg = "";
        Map<String, List<String>> responseHeader = null;

//        Map<String, Object> responseResult = new HashMap<>();
        try {
            TrustManager[] tm = {new MyTrustManager()};
            SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
            sslContext.init(null, tm, new java.security.SecureRandom());
            SSLSocketFactory ssf = sslContext.getSocketFactory();

            URL url = new URL(requestUrl);
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setSSLSocketFactory(ssf);

            if (header != null) {
                header.forEach((key, val) -> conn.addRequestProperty(key, val));
            }

            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod(requestMethod);
            if (null != outputStr) {
                OutputStream outputStream = conn.getOutputStream();
                outputStream.write(outputStr.getBytes("UTF-8"));
                outputStream.close();
            }

            responseCode = conn.getResponseCode();
            responseHeader = conn.getHeaderFields();
            InputStream inputStream;
            if (responseCode >= 400) {
                inputStream = conn.getErrorStream();
            } else {
                inputStream = conn.getInputStream();
            }

            if (inputStream != null) {
                InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
                String str = null;
                StringBuffer buffer = new StringBuffer();
                while ((str = bufferedReader.readLine()) != null) {
                    buffer.append(str);
                }
                body = buffer.toString();
                bufferedReader.close();
                inputStreamReader.close();
                inputStream.close();
            }

            conn.disconnect();

            log.info(" -- url: " + requestUrl);
//            log.info(" -- result: " + body);
        } catch (Exception e) {
            msg = e.getMessage();
            responseHeader.forEach((key, val) -> {
                log.info(" --- " + key + " --- ");
                val.forEach(item -> {
                    log.info(" ---- " + item);
                });
            });
            e.printStackTrace();
        }

        return new HttpResult(responseCode, body, msg, responseHeader);
    }

    public static HttpResult httpRequestMap(String requestUrl, String requestMethod, String outputStr, Map<String, String> header) {
        int responseCode = 30500;//默认系统异常
        String body = "";
        String msg = "";
        Map<String, List<String>> responseHeader = null;
        try {
            URL url = new URL(requestUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();

            if (header != null) {
                header.forEach((key, val) -> conn.addRequestProperty(key, val));
            }
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod(requestMethod.toUpperCase());

            if (null != outputStr) {
                OutputStream outputStream = conn.getOutputStream();
                outputStream.write(outputStr.getBytes("UTF-8"));
                outputStream.close();
            }

            responseCode = conn.getResponseCode();
            responseHeader = conn.getHeaderFields();

            InputStream inputStream;
            if (responseCode >= 400) {
                inputStream = conn.getErrorStream();
            } else {
                inputStream = conn.getInputStream();
            }

            if (inputStream != null) {
                InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
                String str = null;
                StringBuffer buffer = new StringBuffer();
                while ((str = bufferedReader.readLine()) != null) {
                    buffer.append(str);
                }
                body = buffer.toString();
                bufferedReader.close();
                inputStreamReader.close();
                inputStream.close();
            }

            conn.disconnect();

            log.info(" -- url: " + requestUrl);
            log.info(" -- responseCode: " + responseCode);
//            log.info(" -- responseBody: " + body);
        } catch (Exception e) {
            msg = e.getMessage();
            responseHeader.forEach((key, val) -> {
                log.info(" ---head info: " + key + " --- ");
                val.forEach(item -> {
                    log.info(" ---- " + item);
                });
            });

            e.printStackTrace();
        }
        return new HttpResult(responseCode, body, msg, responseHeader);
    }

    private static final long RETRY_SLEEP_TIME_MILLIS = 2000;

    public static CloseableHttpClient createHttpClient(Map<String, String> header) {
        int maxRetries = 3;
        ArrayList<Header> headers = new ArrayList<>();
        header.forEach((key, val) -> {
            headers.add(new BasicHeader(key, val));
        });

        HttpClientBuilder httpBuilder = HttpClients.custom()
                .setDefaultHeaders(headers);
//                .setRetryHandler((exception, executionCount, context) -> {
//                    boolean retry = true;
//
//                    Boolean b = (Boolean) context.getAttribute(ExecutionContext.HTTP_REQ_SENT);
//                    boolean sent = (b != null && b.booleanValue());
//
//                    if (executionCount > maxRetries) {
//                        // Do not retry if over max retry count
//                        retry = false;
////                    } else if (isInList(exceptionBlacklist, exception)) {
////                        // immediately cancel retry if the error is blacklisted
////                        retry = false;
////                    } else if (isInList(exceptionWhitelist, exception)) {
//                        // immediately retry if error is whitelisted
//                        retry = true;
//                    } else if (!sent) {
//                        // for most other errors, retry only if request hasn't been fully
//                        // sent yet
//                        retry = true;
//                    }
//
//                    HttpClientContext clientContext = HttpClientContext.adapt(context);
//                    HttpRequest request = clientContext.getRequest();
//                    HttpResponse response = clientContext.getResponse();
//                    StatusLine statusLine = response.getStatusLine();
//
//                    boolean unauthorized = false;
//                    if (statusLine != null && statusLine.getStatusCode() == 401) {
//                        unauthorized = true;
////                        ServerCaller caller = (ServerCaller) context.getAttribute("FBoxServerCaller");
//                        try {
//                            System.out.println("ServerCaller: try get another token for " + request.toString());
////                            caller.accessToken = caller.tokenManager.getOrUpdateToken(caller.accessToken);
//                            request.setHeader("Authorization", "Bearer " + accessToken);
//                            retry = true;
//                        } catch (Exception e) {
//                            e.printStackTrace();
//                            retry = false;
//                        }
//                    }
//
//                    if (retry && !unauthorized) {
//                        try {
//                            Thread.sleep(RETRY_SLEEP_TIME_MILLIS);
//                        } catch (InterruptedException e) {
//                            e.printStackTrace();
//                        }
//                    } else {
//                        exception.printStackTrace();
//                    }
//                    return retry;
//                });
//        if (proxy != null)
//            httpBuilder = httpBuilder.setProxy(proxy);

        CloseableHttpClient http = httpBuilder.build();
        return http;
    }

}
